@@ -4,6 +4,7 @@
 #
 # backend_itp.py: support for running jobs under the Microsoft ITP (Integrated Training Platform)
 
+import os
 from xtlib.backends.backend_aml import AzureML
 
 class ITP(AzureML):
@@ -18,6 +19,16 @@ class ITP(AzureML):
         return "itp"
 
     def update_estimator(self, estimator, gpu_count, preemption_allowed):
+        # when submitting an ITP job, we do a JIT install of weird AML K8S dependency
+        # (doing it here helps keep pip install for XT working correctly, on client
+        # machine as well as ITP compute node).
+        cmd = "pip install --upgrade --disable-pip-version-check --extra-index-url " + \
+            "https://azuremlsdktestpypi.azureedge.net/K8s-Compute/D58E86006C65 azureml_contrib_k8s"
+
+        fn_log = os.path.expanduser("~/.xt/k8s_install.log")
+        cmd += " > " + fn_log
+        os.system(cmd)
+
         # create Amlk8s config
         from azureml.contrib.core.k8srunconfig import K8sComputeConfiguration
         
@@ -37,7 +48,7 @@ class ITP(AzureML):
 
         # currently, blobfuse is busted for ITP, so we manually upload the logs
         workspace = args["workspace"]
-        cmd = "xt upload {} $jobs/{} --feedback=0 --workspace={}".format(dest_dir, store_path, workspace)
+        cmd = "xt upload {} {} --feedback=0 --workspace={}".format(dest_dir, store_path, workspace)
         self.append(cmds, cmd)
 
     def create_wrapper_script(self, cmd_parts, snapshot_dir, store_data_dir, data_action, 
