@@ -42,23 +42,47 @@ class PoolBackend(BackendBase):
         self.env_vars = None
         self.client = self.core.client
 
+        if pc_utils.is_localhost(compute) and pc_utils.is_windows():
+            self.is_windows = True
+
+    # API 
     def get_name(self):
         return "pool"
 
+    # API 
+    def provides_container_support(self):
+        '''
+        Returns:
+            returns True if docker run command is handled by the backend.
+        '''
+        return False
+
     def adjust_run_commands(self, job_id, job_runs, using_hp, experiment, service_type, snapshot_dir, 
-        args):
+            args):
         '''
         This method is called to allow the backend to inject needed shell commands before the user cmd.  This 
         base implementation does so by generating a new script file and adding it to the snapshot_dir.
         '''
+        gen_for_windows = self.set_gen_for_windows(self.is_windows, args)
+
         store_data_dir, data_action, data_writable, store_model_dir, model_action, model_writable,  \
             storage_name, storage_key = self.get_action_args(args)
 
+        docker_cmd = args["docker_cmd"]
+
         # local or POOL of vm's
-        wrapped_parts = None     # we use same cmd on each box/job
+        fn_wrapped = None     # we use same generated script on each box/job
         data_local = args["data_local"]
         model_local = args["model_local"]
 
+        # if data_local and docker_cmd:
+        #     data_local = "/usr/data"
+
+        if docker_cmd:
+            # best to use blobfuse since it will be available and will be closer to testing remote node 
+            data_local = None
+            model_local = None
+
         for i, box_runs in enumerate(job_runs):
             # wrap the user commands in FIRST RUN of each box (apply data/model actions)
             br = box_runs[0]
@@ -66,6 +90,7 @@ class PoolBackend(BackendBase):
             box_name = box_info.box_name
             box_secret = br["box_secret"]
             actions = box_info.actions
+            node_id = utils.node_id(i)
 
             is_windows = box_info.box_os == "windows"
 
@@ -73,46 +98,70 @@ class PoolBackend(BackendBase):
             cmd_parts = run_specs["cmd_parts"]
             run_name = br["run_name"]
 
-            if not wrapped_parts:
+            if not fn_wrapped:
 
                 # we only do this once (for the first box/job)
                 using_localhost =  pc_utils.is_localhost(box_name, box_info.address)
 
-                # data_local overrides store_data_dir for LOCAL machine
-                if using_localhost and data_local:
-                    store_data_dir = os.path.join(os.path.expanduser(data_local), store_data_dir)
-                    data_action = "use_local"
-                    if not "data" in actions:
-                        actions.append("data")
+                if docker_cmd:
+                    actions = ["data", "model"]
+                else:
+                    # data_local overrides store_data_dir for LOCAL machine
+                    if using_localhost and data_local:
+                        store_data_dir = os.path.join(os.path.expanduser(data_local), store_data_dir)
+                        store_data_dir = file_utils.fix_slashes(store_data_dir, is_linux=not self.gen_for_windows)
 
-                # model_local overrides store_model_dir for LOCAL machine
-                if using_localhost and model_local:
-                    store_model_dir = os.path.join(os.path.expanduser(model_local), store_model_dir)
-                    model_action = "use_local"
-                    if not "model" in actions:
-                        actions.append("model")
+                        data_action = "use_local"
+                        if not "data" in actions:
+                            actions.append("data")
 
-                setup = self.config.get_setup_from_target_def(self.compute_def)
+                    # model_local overrides store_model_dir for LOCAL machine
+                    if using_localhost and model_local:
+                        store_model_dir = os.path.join(os.path.expanduser(model_local), store_model_dir)
+                        model_action = "use_local"
+                        if not "model" in actions:
+                            actions.append("model")
 
-                env_vars = self.get_env_vars_for_box(box_name, box_info, i, box_secret)
-                post_cmds = []
+                setup_name = args["setup"]
+                setup = self.config.get_setup_from_target_def(self.compute_def, setup_name)
 
-                # add env vars to script
-                setter = "@set" if is_windows else "export"
+                env_vars = self.get_env_vars_for_box(box_name, box_info, i, box_secret, args)
+                pre_cmds = []
+
+                # add commands to set ENV VARs
+                self.append_title(pre_cmds, "SET/EXPORT POOL environment vars:")
 
                 for name, value in env_vars.items():
-                    cmd = "{} {}={}".format(setter, name, value)
-                    post_cmds.append(cmd)
+                    self.append_export(pre_cmds, name, value)
 
-                #"xt download before/code --job={} --unzip "
+                use_username = not docker_cmd
+                install_blobfuse = bool(docker_cmd)
 
-                wrapped_parts = super().wrap_user_command(cmd_parts, snapshot_dir, store_data_dir, data_action, 
+                if self.gen_for_windows:
+                    node_index = "%1%"
+                    run_name = "%2%"
+                    homebase = os.getenv("HOMEDRIVE")
+                    cwd = homebase + "\\.xt\\cwd"
+                else:
+                    node_index = "$1"
+                    run_name = "$2"
+                    homebase = "$HOME"
+                    cwd = "$HOME/.xt/cwd"
+
+                fn_wrapped = super().wrap_user_command(cmd_parts, snapshot_dir, store_data_dir, data_action, 
                     data_writable, store_model_dir, model_action, model_writable, storage_name, storage_key, actions, 
-                    is_windows=is_windows, pip_freeze=False, setup=setup, post_setup_cmds=post_cmds, 
-                    run_name=run_name, args=args, nonempty=True)
+                    is_windows=is_windows, sudo_available=False, use_allow_other=False, use_username = use_username, 
+                    setup=setup, pre_setup_cmds=pre_cmds, post_setup_cmds=None, args=args, nonempty=True, 
+                    install_blobfuse=install_blobfuse, node_index=node_index,
+                    run_name=run_name, homebase=homebase, cwd=cwd)   
 
             # we update each box's command
-            run_specs["cmd_parts"] = wrapped_parts
+            script_part = "{} {} {}".format(os.path.basename(fn_wrapped), i, run_name)
+            if self.is_windows:
+                sh_parts = [script_part]
+            else:
+                sh_parts = ['/bin/bash', '--login', script_part]
+            run_specs["cmd_parts"] = sh_parts
 
     def submit_job(self, job_id, job_runs, workspace, compute_def, resume_name, 
            repeat_count, using_hp, runs_by_box, experiment, snapshot_dir, controller_scripts, args):
@@ -123,12 +172,12 @@ class PoolBackend(BackendBase):
 
         if not fake_submit:
             # first pass - ensure all boxes are NOT already running the controller
-            for i, box_runs in enumerate(job_runs):
-                box_info = box_runs[0]["box_info"]
+            # for i, box_runs in enumerate(job_runs):
+            #     box_info = box_runs[0]["box_info"]
 
-                box_name = box_info.box_name
-                box_addr = box_info.address
-                port = constants.CONTROLLER_PORT
+            #     box_name = box_info.box_name
+            #     box_addr = box_info.address
+            #     port = constants.CONTROLLER_PORT
 
                 # if self.core.client.is_controller_running(box_name, box_addr, port):
                 #     errors.config_error("XT controller already running on box: " + box_name)
@@ -155,39 +204,46 @@ class PoolBackend(BackendBase):
         box_os = box_info.box_os
         is_box_windows = (box_os == "windows")
         
+        run_data = run_data_list[0]
+        run_name = run_data["run_name"]
+
         if pc_utils.is_localhost(box_addr=box_addr):
             psm_client = LocalPsmClient()
         else:
             psm_client = RemotePsmClient(box_addr, is_box_windows)
 
-        psm_client.start_psm_if_needed()
+        psm_client.restart_psm_if_needed()
         #print("psm created for box: " + box_addr)
 
-        team = self.config.get("general", "xt-team-name")
+        service_set_name = self.config.get("service-set")
         node_id = utils.node_id(box_index)
 
         cwd_dir = os.path.expanduser(constants.CWD_DIR)
         fn_src_zip = file_utils.path_join(cwd_dir, constants.CODE_ZIP_FN)
+        workspace = args["workspace"]
 
-        fn_entry = psm_client.enqueue(team, job_id, node_id, fn_src_zip)
-        run_data = run_data_list[0]
+        fn_entry = psm_client.enqueue(service_set_name, workspace, job_id, run_name, box_index, fn_src_zip)
 
         service_node_info = {"fn_entry": fn_entry, "box_addr": box_addr, "box_os": box_os, 
-            "box_name": box_name, "job_id": job_id, "run_name": run_data["run_name"]}
+            "box_name": box_name, "job_id": job_id, "run_name": run_name}
 
         fb.feedback("submitted", is_final=True) 
 
         return service_node_info
 
-    def get_env_vars_for_box(self, box_name, box_info, nodex_index, box_secret):
+    def get_env_vars_for_box(self, box_name, box_info, nodex_index, box_secret, args):
         #box_secrets.set_secret(box_name, box_secret)
 
         box_os = box_info.box_os
         max_runs = box_info.max_runs
 
         node_id = "node" + str(nodex_index)
-        env_vars = {}
-        scriptor.add_controller_env_vars(env_vars, self.config, box_secret, node_id)
+
+        env_vars = dict(args["env_vars"])
+        scriptor.add_controller_env_vars(env_vars, self.config, box_secret, None)
+
+        # XT_RUN_NAME must be set thru environment variable so that script can be shared
+        #env_vars["XT_RUN_NAME"] = run_name
 
         return env_vars
 
@@ -259,12 +315,13 @@ class PoolBackend(BackendBase):
         #   - use batch_client "cancel node" if controller not available
 
         # we build service-based box names to have 3 parts
-        active_jobs = self.get_active_jobs()
+        active_jobs = self.get_active_jobs(workspace)
         cancel_results = []
         if active_jobs:
             for job_record in active_jobs:
                 service_job_info = job_record["service_job_info"]
                 service_info_by_node = job_record["service_info_by_node"]
+                
                 for node, node_service_info in service_info_by_node.items():
                     if node_service_info.get("run_name") in run_names:
                         cancel_result = self.cancel_node(node_service_info)
@@ -304,7 +361,7 @@ class PoolBackend(BackendBase):
 
         return cancel_results_by_box
 
-    def cancel_runs_by_user(self, box_name):
+    def cancel_runs_by_user(self, ws_name, box_name):
         '''
         Args:
             box_name: the name of the box the runs ran on (pool service)
@@ -312,7 +369,7 @@ class PoolBackend(BackendBase):
             cancel_results: a list of kill results records 
                 (keys: workspace, run_name, exper_name, killed, status, before_status)
         '''
-        active_jobs = self.get_active_jobs()
+        active_jobs = self.get_active_jobs(ws_name)
         cancel_results = []
         if active_jobs:
             for job_record in active_jobs:
@@ -361,10 +418,15 @@ class PoolBackend(BackendBase):
         if new_text:
             service_status = "running"
         else:
-            # refresh STATUS (takes 1.5, 4.5, or 6 secs)
-            #start = time.time()
-            service_status = psm_client.get_status(fn_entry)
-            #utils.report_elapsed(start, "status")
+            # refresh STATUS 
+            # we retry "completed" a few times to avoid a false positive when transitioning between the 
+            # wrapper script and the controller app
+            count = 10 if start_offset == 0 else 3
+
+            for _ in range(count):
+                service_status = psm_client.get_status(fn_entry)
+                if service_status != "completed":
+                    break
 
         simple_status = self.get_simple_status(service_status)
 
@@ -434,7 +496,7 @@ class PoolBackend(BackendBase):
 
         return queue
 
-    def get_active_jobs(self):
+    def get_active_jobs(self, ws_name):
         ''' return a list of job_id's running on this instance of Azure Batch '''
         mongo = self.client.store.get_mongo()
 
@@ -449,6 +511,23 @@ class PoolBackend(BackendBase):
 
         fields_dict = {"job_id": 1, "service_info_by_node": 1, "service_job_info": 1}
 
-        job_records = mongo.get_info_for_jobs(filter_dict, fields_dict)
+        job_records = mongo.get_info_for_jobs(ws_name, filter_dict, fields_dict)
 
         return job_records
+
+    def get_log_save_cmd(self, output_dir):
+
+        # 3rd argument to script is the path of entry's log file
+        if self.gen_for_windows:
+            cmd = "cp -v %3% {}".format(output_dir)
+        else:
+            cmd = "cp -v $3 {}".format(output_dir)
+        return cmd
+
+   # API call
+    def add_service_log_copy_cmds(self, cmds, dest_dir, store_path, args):
+        # $3 is the path to the local log file for the node
+        if self.gen_for_windows:
+            self.append(cmds, "xcopy %3% {}\\".format(dest_dir))
+        else:
+            self.append(cmds, "mkdir -p {} && cp $3 {}".format(dest_dir, dest_dir))
