@@ -163,7 +163,7 @@ class CmdCore():
     def get_runs_by_boxes_from_job(self, job_id):
         cancel_results_by_boxes = {}
 
-        if not str(job_id).startswith("job"):
+        if not job_helper.is_job_id(str(job_id)):
             errors.syntax_error("not a valid job id: " + str(job_id))
 
         #console.print("job_id=", job_id)
@@ -183,7 +183,7 @@ class CmdCore():
                 if utils.is_azure_batch_box(box_name):
                     job_id, service, node_index = box_name.split("-")
                     azure_batch = AzureBatch(core=self)
-                    cancel_results, _ = azure_batch.cancel_job_node(self.store, job_id, node_index, run_datas)
+                    cancel_results, _ = azure_batch.cancel_job_node(self.store, ws_name, job_id, node_index, run_datas)
                 else:
                     # connect to specified box
                     self.client.change_box(box_name)
@@ -301,6 +301,7 @@ class CmdCore():
 
         context.pool = args["pool"]
         context.node_index = node_index
+        context.node_count = node_count
         context.compute = args["target"]
         context.service_type = args["service_type"]
 
@@ -313,40 +314,11 @@ class CmdCore():
         context.run_script = run_specs["run_script"]
         context.parent_script = run_specs["parent_script"]
 
-        # for helping docker login to user's Azure Container Registry
-        is_docker = (args["docker"] != None)
-        # if cmd_parts:
-        #     is_docker = (cmd_parts[0] == "docker") or (cmd_parts[0] == "sudo" and cmd_parts[1] == "docker")
-
-        #registry = config.get("environment", "registry", suppress_warning=True)
-        registry = None
         compute_def = args["compute_def"]
-        if compute_def and "docker" in compute_def:
-            docker_name = compute_def["docker"]
-            docker_def = self.config.get_docker_def(docker_name)
-            if docker_def and "registry" in docker_def:
-                registry = docker_def["registry"]
-
-        if registry:
-            registry_creds = config.get("external-services", registry)
-            needs_login = is_docker and utils.safe_value(registry_creds, "login")
-            login_server = utils.safe_value(registry_creds, "login-server")
-            username = utils.safe_value(registry_creds, "username")
-            password = utils.safe_value(registry_creds, "password")
-        else:
-            needs_login = False
-            login_server = None
-            username = None
-            password = None
-
-        context.docker_login = needs_login
-        context.docker_server = login_server
-        context.docker_username = username
-        context.docker_password = password
-
         context.username = self.config.get("general", "username")
 
-        setup = self.config.get_setup_from_target_def(compute_def)
+        setup_name = args["setup"]
+        setup = self.config.get_setup_from_target_def(compute_def, setup_name)
         activate_cmd = utils.safe_value(setup, "activate")
         context.activate_cmd = activate_cmd
 
@@ -405,7 +377,15 @@ class CmdCore():
         context.store_code_path = config.get_storage_provider_code_path(store_creds)
 
         mongo_creds, mongo_name = self.config.get_mongo_creds()
-        context.mongo_conn_str = mongo_creds["mongo-connection-string"]
+
+        ss_info = self.config.get_service_set_info()
+        storage_tracking = utils.safe_value(ss_info, "track-storage")
+
+        if storage_tracking:
+            # no mongo is available for run to use (everything is logged to storage)
+            context.mongo_conn_str = ""
+        else:
+            context.mongo_conn_str = mongo_creds["mongo-connection-string"]
 
         context.shell_launch_prefix = box_info.shell_launch_prefix
         
@@ -457,7 +437,8 @@ class CmdCore():
         return target, before_parts, after_parts
 
     def docker_login(self, server, username, password):
-        exit_code, output = process_utils.sync_run(["docker", "login", server, "--username", username, "--password", password],  capture_output=True, shell=False, report_error=True)
+        exit_code, output = process_utils.sync_run(["docker", "login", server, "--username", username, "--password", password],  
+            capture_output=True, shell=False, report_error=True)
         return output
 
     def docker_logout(self, server):
@@ -488,10 +469,10 @@ class CmdCore():
         return count
 
     def adjust_job_for_direct_run(self, job_id, job_runs, cmds, using_hp, experiment, service_type, 
-            snapshot_dir, search_style, args):
+            bootstrap_dir, search_style, args):
 
         # write 1st file to SNAPSHOT (first run's context file)
-        fn_run_context = snapshot_dir + "/" + constants.FN_RUN_CONTEXT 
+        fn_run_context = bootstrap_dir + "/" + constants.FN_RUN_CONTEXT 
         file_utils.ensure_dir_exists(file=fn_run_context)
         
         box_runs = job_runs[0]
@@ -507,8 +488,8 @@ class CmdCore():
 
         return [fn_run_context]
             
-    def adjust_job_for_controller_run(self, job_id, job_runs, cmds, using_hp, experiment, service_type, 
-            snapshot_dir, search_style, args):
+    def adjust_job_for_controller_run(self, job_id, job_runs, cmds, runsets, using_hp, experiment, service_type, 
+            bootstrap_dir, search_style, args):
         ''' 
         submit direct job:
             - backend.commands: some internal prep cmds and the run command(s) specified by the user
@@ -536,19 +517,20 @@ class CmdCore():
             job_runs[i] = new_box_runs
 
         # write 1st file to SNAPSHOT (MRC file)
-        fn_context = snapshot_dir + "/" + constants.FN_MULTI_RUN_CONTEXT 
+        fn_context = bootstrap_dir + "/" + constants.FN_MULTI_RUN_CONTEXT 
         file_utils.ensure_dir_exists(file=fn_context)
-        
-        mrc_data = { "context_by_nodes": context_by_nodes, "cmds": cmds, "search_style": search_style }
+        workspace = args["workspace"]
+
+        mrc_data = { "context_by_nodes": context_by_nodes, "cmds": cmds, "runsets": runsets, "search_style": search_style }
         text = json.dumps(mrc_data, indent=4)
         with open(fn_context, "wt") as tfile:
             tfile.write(text)
 
         # also write the MRC file to the job store (to support wrapup of runs after job is cancelled)
-        self.store.create_job_file(job_id, constants.FN_MULTI_RUN_CONTEXT, text)
+        self.store.create_job_file(workspace, job_id, constants.FN_MULTI_RUN_CONTEXT, text)
 
         # write 2rd file to SNAPSHOT
-        fn_script = snapshot_dir + "/" + constants.PY_RUN_CONTROLLER
+        fn_script = bootstrap_dir + "/" + constants.PY_RUN_CONTROLLER
         is_aml = (service_type == "aml")
 
         with open(fn_script, "wt") as outfile:
@@ -605,50 +587,40 @@ class CmdCore():
 
         console.diag("before upload of {} to job".format(upload_type.upper()))
   
-        copied_files = capture.capture_before_files_zip(self.store, source_dir=source_dir, omit_files=omit_list, store_dest=store_dir,
-            rerun_name=resume_name, job_id=job_id, zip_before=zip_type, remove_prefix_len=remove_prefix_len, upload_type=upload_type)
+        service_type = args["service_type"]
+        workspace = args["workspace"]
 
-        #fb.feedback("{} target file(s) uploaded to {}".format(len(copied_files), dest_name))
-        fb.feedback("")    # just add comma to indicte current op has completed
+        copied_files = capture.capture_before_files_zip(self.store, source_dir=source_dir, omit_files=omit_list, store_dest=store_dir,
+            rerun_name=resume_name, job_id=job_id, zip_before=zip_type, remove_prefix_len=remove_prefix_len, upload_type=upload_type, 
+            service_type=service_type, ws_name=workspace)
 
         console.diag("after upload of {} to job".format(upload_type.upper()))
         return copied_files
 
-    def create_backend(self, compute, username=None, cluster=None, vc=None):
+    def create_backend(self, compute, compute_def=None, username=None):
         '''
-        TODO: replace "cluster" and "vc" arguments with regularized 
-        "compute_def" argument.
+        args:
+            compute: name of compute target
+            compute_def: properties of compute (optional)
+            username: the name of the user (from cmd option)
+        description:
+            create helper for backend compute service (pool, philly, batch, aml)
         '''
-
-        compute_def = self.config.get_compute_def(compute)
-        service = compute_def["service"]
-        service_type = self.config.get_service_type(service)
+        if not compute_def:
+            compute_def = dict(self.config.get_compute_def(compute))
 
         if not username:
             # not specified as an option; default to config file
             username = self.config.get("general", "username")
 
-        if not cluster:
-            # not specified as an option; default to compute-def
-            if "cluster" in compute_def:
-                cluster = compute_def["cluster"]
-
-        if not vc:
-            # not specified as an option; default to compute-def
-            if "vc" in compute_def:
-                vc = compute_def["vc"]
+        service = compute_def["service"]
+        service_type = self.config.get_service_type(service)
 
         # get code_path for service_name from compute_providers
         service_name = "pool" if service_type == "local" else service_type
         backend_ctr = self.config.get_provider_class_ctr("compute", service_name)
 
-        # instantiate the associated backend class
-        arg_dict = {"cluster": cluster, "vc": vc}
-
-        '''
-        TODO: eliminate "arg_dict" argument for backend CTR by regularizing all options into "compute_def".
-        '''
-        backend = backend_ctr(compute=compute, compute_def=compute_def, core=self, config=self.config, username=username, arg_dict=arg_dict)
+        backend = backend_ctr(compute=compute, compute_def=compute_def, core=self, config=self.config, username=username, arg_dict=None)
         return backend
 
     def get_box_run_status_inner(self, box_name, ws=None, run_name=None, stage_flags=""):
