@@ -467,7 +467,7 @@ class Trainer():
 
     def init_dirs(self, args):
         # set mnt_output_dir (using environment variable setting from xt)
-        mnt_output_dir = os.getenv("XT_OUTPUT_MNT", "output")
+        mnt_output_dir = os.getenv("XT_OUTPUT_DIR", "output")
         mnt_output_dir = os.path.expanduser(mnt_output_dir)
         file_utils.ensure_dir_exists(mnt_output_dir)
         print("writing mnt_output to: " + mnt_output_dir)
@@ -726,7 +726,7 @@ class Trainer():
             outfile.write("This is a file that should be omitted from AFTER upload\n")
             outfile.write("end of junk file\n")
 
-        if run:
+        if run and run.store.mongo:
             # ensure we close all logging
             print("miniMNST: mongo call stats")
             run.store.mongo.print_call_stats()
