@@ -409,4 +409,8 @@ def rel_path(path, to_path=None):
     if rp == "":
         rp = "."
 
-    return rp
\ No newline at end of file
+    return rp
+
+def get_ext(path):
+    _, file_ext = os.path.splitext(path)
+    return file_ext
\ No newline at end of file
