@@ -28,7 +28,6 @@ class XTConfig():
         self.data = self.read_config(fn)
         self.explicit_options = {}
         self.vault = None
-        self.mini_mode = False
 
     def name_exists(self, group, name):
         return group in self.data and name in self.data[group]
@@ -131,9 +130,10 @@ class XTConfig():
 
             # create our vault manager
             vault_url = self.get_vault_url()
-            team_name = self.get("general", "xt-team-name")
 
-            self.vault = xt_vault.XTVault(vault_url, team_name)
+            service_set_name = self.get("service-set")
+
+            self.vault = xt_vault.XTVault(vault_url, service_set_name)
 
             authentication = self.get("general", "authentication")
             self.vault.init_creds(authentication)
@@ -149,13 +149,13 @@ class XTConfig():
             assert section and prop_name
             text = self.vault.get_secret(section)
 
-        if "$username" in text:
+        if text and "$username" in text:
             ev_user = "USERNAME" if pc_utils.is_windows() else "USER"
             username = os.getenv(ev_user, os.getenv("XT_USERNAME"))
             username = username if username else ""
             text = text.replace("$username", username)
 
-        if "$current_conda_env" in text:
+        if text and "$current_conda_env" in text:
             conda = os.getenv("CONDA_DEFAULT_ENV")
             if conda:
                 text = text.replace("$current_conda_env", conda)
@@ -210,11 +210,13 @@ class XTConfig():
 
         return setup_def
 
-    def get_setup_from_target_def(self, target_def):
+    def get_setup_from_target_def(self, target_def, setup_name=None):
         setup_def = None
 
-        if target_def and "setup" in target_def:
+        if not setup_name and target_def and "setup" in target_def:
             setup_name = target_def["setup"]
+
+        if setup_name:
             setup_def = self.get("setups", setup_name, suppress_warning=True)
 
         return setup_def
@@ -234,23 +236,23 @@ class XTConfig():
         return service
 
     def get_compute_def(self, target_name): 
-        target = self.get("compute-targets", target_name, suppress_warning=True)
+        compute_def = self.get("compute-targets", target_name, suppress_warning=True)
 
-        if not target:
+        if not compute_def:
             # is this target a box name?
             box_info = self.get("boxes", target_name, suppress_warning=True)
             if not box_info:
                 errors.config_error("target '{}' must be defined in the [compute-targets] section of XT config file (or be box name)".format(target_name))
             # make box look like a target
-            target = {"service": "pool", "boxes": [target_name]}
+            compute_def = {"service": "pool", "boxes": [target_name]}
 
             # use setup from first box 
             if "setup" in box_info:
-                target["setup"] = box_info["setup"]
+                compute_def["setup"] = box_info["setup"]
 
-        target["name"] = target_name
+        compute_def["name"] = target_name
         #self.expand_symbols_in_creds(target, target_name)
-        return target        
+        return compute_def        
 
     def get_external_service_from_target(self, target_name):
         target = self.get_compute_def(target_name)
@@ -271,9 +273,10 @@ class XTConfig():
 
     def get_storage_creds(self):
         # validate STORAGE service
-        storage_name = self.get("xt-services", "storage", suppress_warning=True)
+        ss_info = self.get_service_set_info()
+        storage_name = utils.safe_value(ss_info, "storage")
         if not storage_name:
-            errors.config_error("'storage' must be set in [xt-services] section of XT config file")
+            errors.config_error("'storage' property must be set in service-set '{}'".format(ss_info["name"]))
 
         # validate STORAGE_NAME credentials
         storage_creds = self.get("external-services", storage_name, suppress_warning=True)
@@ -286,9 +289,10 @@ class XTConfig():
 
     def get_mongo_creds(self):
         # validate MONGO service
-        mongo_name = self.get("xt-services", "mongo", suppress_warning=True)
+        ss_info = self.get_service_set_info()
+        mongo_name = utils.safe_value(ss_info, "mongo")
         if not mongo_name:
-            errors.config_error("'mongo' must be set in [xt-services] section of XT config file")
+            errors.config_error("'mongo' property must be set in service-set '{}'".format(ss_info["name"]))
 
         # validate MONGO credentials
         mongo_creds = self.get("external-services", mongo_name, suppress_warning=True)
@@ -300,9 +304,11 @@ class XTConfig():
 
     def get_vault_url(self):
         # validate VAULT service
-        vault_name = self.get("xt-services", "vault", suppress_warning=True)
+        ss_info = self.get_service_set_info()
+        vault_name = utils.safe_value(ss_info, "vault")
         if not vault_name:
-            errors.config_error("'vault' property must be set in [xt-services] section of XT config file")
+            # empty value name is acceptable for some service sets
+            return None
 
         # validate VAULT credentials
         vault_creds = self.get("external-services", vault_name, suppress_warning=True)
@@ -366,18 +372,86 @@ class XTConfig():
         code_path = providers[name]
         return utils.get_class_ctr(code_path)
 
-    # def get_regularized_compute_def(self, compute, opt_names, explicit, args):
-    #     compute_def = self.get_compute_def(compute)  
-    #     if compute_def:
-    #         for name in opt_names:
-    #             under_name = name.replace("-", "_")
+    def get_docker_info(self, target: str, docker_name: str, required: bool):
+        docker_image = None
+        login_server = None
+        docker_registry = None
 
-    #             if name in explicit:
-    #                 sd[name] = args[under_name]
-    #             elif name in sd:
-    #                 args[under_name] = sd[name]
+        compute_def = self.get_compute_def(target)
 
-    #     return compute_def
+        if not docker_name:
+            docker_name = utils.safe_value(compute_def, "docker")
+            
+        if docker_name:
+            env = self.get("dockers", docker_name, default_value=None)
+            if not env:
+                errors.config_error("docker '{}' definition not found in XT config file".format(docker_name))
+
+            docker_image = utils.safe_value(env, "image")
+            if not docker_image:
+                errors.config_error("Definition for docker '{}' is missing the required 'image' property".format(docker_name))
+
+            registry_name = utils.safe_value(env, "registry")
+            # if not registry_name:
+            #     errors.config_error("Definition for docker '{}' is missing the required 'registry' property".format(docker_name))
+
+            if registry_name:
+                docker_registry = self.get("external-services", registry_name, default_value=None)
+                if not docker_registry:
+                    errors.config_error("Definition for docker registry '{}' is missing in the external-services section".format(registry_name))
+
+                login_server = utils.safe_value(docker_registry, "login-server")
+                if not login_server:
+                    errors.config_error("Definition for registry '{}' is missing the required 'login-server' property".format(registry_name))
+        elif required:
+            # all philly compute targets must specify a docker image
+            errors.config_error("Philly compute target '{}' is missing the required 'docker' property (in XT config file)".format(target))
+
+        return docker_image, login_server, docker_registry
+
+    def get_target_desc(self, target, backend):
+        '''
+        returns a string describing the target (for logging and monitoring)
+        '''
+        backend_name = backend.get_name()
+        compute_def = backend.compute_def
+
+        if backend_name == "philly":
+            cluster = backend.cluster
+            vc = backend.vc
+            queue = backend.queue
+            target_str = "target={}, backend={}, cluster={}, vc={}, queue={}".format(target, backend_name, cluster, vc, queue)
+
+        elif backend_name in ["aml", "itp"]:
+            service = compute_def["service"]
+            compute = compute_def["compute"]
+            target_str = "target={}, backend={}, service={}, compute={}".format(target, backend_name, service, compute)
+
+        elif backend_name == "batch":
+            service = compute_def["service"]
+            target_str = "target={}, backend={}, service={}".format(target, backend_name, service)
+
+        elif backend_name == "pool":
+            target_str = "target={}, backend={}".format(target, backend_name)
+
+        else:
+            errors.internal_error("unknown backend name=", backend_name)
+
+        return target_str
+
+    def get_service_set_info(self, service_set_name=None):
+        if not service_set_name:
+            service_set_name = self.get("service-set", suppress_warning=True)
+            
+        if not service_set_name:
+            errors.config_error("XT config file is missing required property: service-set")
+        
+        service_set_info = self.get("service-sets", service_set_name, suppress_warning=True)
+        if not service_set_info:
+            errors.config_error("XT config file is missing definition for service-set: {}".format(service_set_name))
+
+        service_set_info["name"] = service_set_name
+        return service_set_info
 
 # flat functions
 
@@ -386,7 +460,7 @@ def load_yaml(fn):
         data = yaml.safe_load(file) #, Loader=yaml.FullLoader)
     return data
 
-def merge_configs(config, overrides):
+def _merge_configs(config, overrides):
     # note: a simple dict "update()" is too blunt; we need a fine-grained key/value update
     config_data = config.data
 
@@ -395,8 +469,11 @@ def merge_configs(config, overrides):
         if not section_name in config_data:
             config_data[section_name] = {}
 
-        # section_value could be a dict or an array
-        if isinstance(section_value, list):
+        # section_value could be a simple str
+        if isinstance(section_value, str):
+            # just replace list as a single value
+            config_data[section_name] = section_value
+        elif isinstance(section_value, list):
             # just replace list as a single value
             config_data[section_name] = section_value
         else:
@@ -414,9 +491,39 @@ def merge_configs(config, overrides):
                     #console.print("overridding: [{}] {} = {}".format(section_name, key, value))
                     config_data[section_name][key] = value    
 
-def get_merged_config(create_if_needed=True, local_overrides_path=None, suppress_warning=False, mini=False):
+def _apply_override(validator, config, schema, fn_schema, fn_override):
+    
+    overrides = XTConfig(fn=fn_override, create_if_needed=False)
 
-    config = XTConfig()     # create_if_needed=create_if_needed)
+    if overrides.data:
+        # validate the local overrides config file
+        validator.validate(schema, fn_schema, overrides.data, fn_override, False)
+
+        # merge the overrides config with the default config
+        _merge_configs(config, overrides)
+    
+def _apply_basic_mode_rules(config):
+    sc = os.getenv("XT_STORE_CREDS")
+    mc = os.getenv("XT_MONGO_CONN_STR")
+    if sc and mc:
+        # we are running on compute node (launched by script)
+        #console.print("XT: detected run on compute node; setting advanced mode ON")
+        config.set("general", "advanced-mode", value=True)
+
+    if not config.get("general", "advanced-mode"):
+        # single workspace
+        # leave the workspace name from FACTORY
+        #config.data["general"]["workspace"] = "txt"
+
+        # single target
+        # TODO: are we going to support "local" for basic mode?  
+        ss_info = config.get_service_set_info()
+        ss_info["target"] = "batch"
+
+def get_merged_config(create_if_needed=True, local_overrides_path=None, suppress_warning=False):
+
+    # load the FACTORY config
+    config = XTConfig()    
 
     # load the schema for XT CONFIG files
     fn_schema = os.path.join(file_utils.get_xtlib_dir(), "helpers", "xt_config_schema.yaml")
@@ -426,52 +533,19 @@ def get_merged_config(create_if_needed=True, local_overrides_path=None, suppress
     validator = Validator()
     validator.validate(schema, fn_schema, config.data, config.fn, True)
 
-    # apply local override file, if present
-    fn_overrides = local_overrides_path if local_overrides_path else constants.CONFIG_FN
-    fn_overrides = os.path.realpath(fn_overrides)
-    
-    sc = os.getenv("XT_STORE_CREDS")
-    mc = os.getenv("XT_MONGO_CONN_STR")
-    if sc and mc:
-        # we are running on compute node (launched by script)
-        console.print("XT: detected run on compute node; setting mini_mode=False")
-        config.mini_mode = False
-    else:
-        # get mini_mode value from default config (modified further below)
-        config.mini_mode = not config.get("general", "advanced-mode")
-        if config.mini_mode:
-            suppress_warning = True
+    # is there a GLOBAL config?
+    fn_global = os.path.expanduser(os.getenv("XT_GLOBAL_CONFIG", "~/.xt/xt_config.yaml"))
+    if os.path.isfile(fn_global):    
+        _apply_override(validator, config, schema, fn_schema, fn_global)
 
-    if os.path.exists(fn_overrides):
-        overrides = XTConfig(fn=fn_overrides, create_if_needed=False)
-        #console.print("config.data['apps']=", config.data['apps'].keys())
+    # is there a LOCAL config?
+    fn_local = local_overrides_path if local_overrides_path else constants.CONFIG_FN
+    if os.path.isfile(fn_local):    
+        _apply_override(validator, config, schema, fn_schema, fn_local)
 
-        if not overrides.data:
-            console.warning("local xt_config.yaml file contains no properties")
-        else:
-            # validate the local overrides config file
-            validator.validate(schema, fn_schema, overrides.data, fn_overrides, False)
+    # apply special BASIC MODE rules
+    _apply_basic_mode_rules(config)
 
-            # allow overrides to override the mini_mode flag
-            if not (sc and mc):
-                config.mini_mode = not overrides.get("general", "advanced-mode", suppress_warning=True)
-
-            # hardcoded MINI options (can be overwritten by local confile file)
-            if config.mini_mode:
-                # single workspace
-                config.data["general"]["workspace"] = "txt"
-
-                # single target
-                config.data["xt-services"]["target"] = "batch"
-
-            # merge the overrides config with the default config
-            merge_configs(config, overrides)
-           
-    else:
-        if not suppress_warning:
-            console.print("warning: no local config file found")
-    
     console.detail("after loading/validation of merged config files")
-    
     return config
 
