@@ -11,6 +11,8 @@ import json
 from interface import implements
 
 from xtlib import console
+from xtlib import constants
+from xtlib.hparams import hp_helper
 from xtlib.hparams.hp_search_interface import HpSearchInterface
 
 # TERMINOLOGY
@@ -26,13 +28,6 @@ from xtlib.hparams.hp_search_interface import HpSearchInterface
 # config - Short term for configuration.
 # RunSet - The class encapsulating a list of runs that share the same configuration.
 
-# globals
-USE_SUCCESS_RATE = False
-SUCCESS_RATE_THRESHOLD = 1
-HELDOUT_TESTING = False
-
-TOTAL_STEPS = 100
-REPORTING_INTERVAL = 10
 
 dgd_rand = random.Random(time.time())  # For 'truly' random hyperparameter selection.
 
@@ -58,18 +53,14 @@ class DGDSearch(implements(HpSearchInterface)):
 
 class MetricReport(object):
     def __init__(self, metric_dict, primary_metric):
-        #self.time = float(metric_dict["sec"])
-        #self.steps = int(metric_dict["steps"])
-        global USE_SUCCESS_RATE
-        if USE_SUCCESS_RATE:
-            self.score = float(metric_dict["RSR"])
-            self.sample_efficiency = float(metric_dict[primary_metric])   # "SE"])
-        else:
-            self.score = float(metric_dict[primary_metric])   # "reward"])
+        self.score = float(metric_dict[primary_metric])
 
 
 class RunSet(object):
     def __init__(self, hp_id__setting_id__list, config_str):
+        # sort to ensure a consistent order
+        hp_id__setting_id__list.sort()
+
         self.hp_id__setting_id__list = hp_id__setting_id__list
         self.config_str = config_str
         self.runs = []
@@ -86,12 +77,13 @@ class RunSet(object):
 
 class Run(object):
     def __init__(self, run_data, primary_metric):
+        self.run_name = run_data["run_name"]   # for debugging
         self.hpname_hpvalue_list = []
         self.reset_metrics()
 
-        # process "hparams" subrecord
+        # process hparams subrecord
         if "hparams" in run_data:
-            hparams = run_data["hparams"]     #  event_dict["data"]
+            hparams = run_data["hparams"]    
             for prop, value in hparams.items():
                 self.hpname_hpvalue_list.append( (prop, value) )
 
@@ -109,18 +101,12 @@ class Run(object):
     def add_metric_report(self, metrics, primary_metric):
         metric_report = MetricReport(metrics, primary_metric)
         self.metric_reports.append(metric_report)
-        global USE_SUCCESS_RATE
-        if USE_SUCCESS_RATE:
-            self.overall_score = metric_report.sample_efficiency
-        else:
-            self.overall_score += metric_report.score
+        self.overall_score += metric_report.score
 
     def normalize_overall_score(self):
         num_reports = len(self.metric_reports)
-        global USE_SUCCESS_RATE
-        if not USE_SUCCESS_RATE:
-            if num_reports > 0:
-                self.overall_score /= num_reports
+        if num_reports > 0:
+            self.overall_score /= num_reports
 
 
 class HyperparameterSetting(object):
@@ -227,28 +213,21 @@ class DGD(object):
                 in_hp_section = record["in_hp_section"]
                 comment = record["comment"]
 
+            # all our values are string but we only accept
+            # hparams with a list of values (containing commas)
+            if not "," in value_string:
+                console.print("skipping hparam: {}={}".format(name_string, value_string))
+                continue
+
+            console.print("adding hparam: {}={}".format(name_string, value_string))
+
             hp = Hyperparameter(len(self.hparams), name_string, value_string, comment, in_hp_section)
             self.hparams.append(hp)
             self.name_hparam_dict[name_string] = hp
 
-        global USE_SUCCESS_RATE, SUCCESS_RATE_THRESHOLD, HELDOUT_TESTING
-
-        if "USE_SUCCESS_RATE" in self.name_hparam_dict.keys():
-            USE_SUCCESS_RATE = self.name_hparam_dict["USE_SUCCESS_RATE"].single_value
-        if "SUCCESS_RATE_THRESHOLD" in self.name_hparam_dict.keys():
-            SUCCESS_RATE_THRESHOLD = self.name_hparam_dict["SUCCESS_RATE_THRESHOLD"].single_value
-        if "HELDOUT_TESTING" in self.name_hparam_dict.keys():
-            HELDOUT_TESTING = self.name_hparam_dict["HELDOUT_TESTING"].single_value
-
     def create_run_objects(self, run_reports, primary_metric):
         self.runs = []
 
-        # if "TOTAL_STEPS" in self.name_hparam_dict:
-        #     TOTAL_STEPS = self.name_hparam_dict["TOTAL_STEPS"].single_value
-        # if "REPORTING_INTERVAL" in self.name_hparam_dict:
-        #     REPORTING_INTERVAL = self.name_hparam_dict["REPORTING_INTERVAL"].single_value
-        # self.max_reports_per_run = TOTAL_STEPS // REPORTING_INTERVAL
-
         for record in run_reports:
             if self.unit_test:
                 run = Run(json.loads(record))
@@ -259,12 +238,17 @@ class DGD(object):
                 continue  # Skip parent runs.
             # Try to assemble a configuration string for this run.
             hp_id__setting_id__list = []
-            for hp_name, hp_value in run.hpname_hpvalue_list:
 
+            # remove any reported hparams not found in search (allowed in XT)
+            hp_list = run.hpname_hpvalue_list
+            search_keys = list(self.name_hparam_dict.keys())
+            new_list = [(hp_name, hp_value) for hp_name, hp_value in hp_list if hp_name in search_keys]
+            run.hpname_hpvalue_list = new_list
+
+            for hp_name, hp_value in run.hpname_hpvalue_list:
                 # we no longer require that all run-reported hparams are present in the hp-config file
-                #assert hp_name in self.name_hparam_dict.keys()  # hparams must not be removed from config.txt
-                if not hp_name in self.name_hparam_dict:
-                    continue    
+                # assert is here to ensure above filtering of run.hpname_hpvalue_list worked correctly
+                assert hp_name in self.name_hparam_dict.keys()  
 
                 hparam = self.name_hparam_dict[hp_name]
                 if not hparam.in_hp_section:  # Ignore controls.
@@ -273,6 +257,7 @@ class DGD(object):
                 if hp_value not in hparam.value_setting_dict:
                     # REVIEW: why are we landing here?
                     ignore_run = True
+                    console.print("skipping run: {} (hp_value={}, dict={})".format(run.run_name, hp_value, hparam.value_setting_dict))
                     break  # Skip this run. Its value must have been removed from config.txt.
 
                 if hparam.has_multiple_values:  # Filter to HPs with multiple values.
@@ -293,19 +278,16 @@ class DGD(object):
             runset = self.configstr_runset_dict[config_str]
             runset.runs.append(run)
 
+            #console.print("added run: {}".format(run.run_name))
+
     def finalize_runsets(self):
-        global USE_SUCCESS_RATE
         for runset in self.runsets:
             runset.num_runs = len(runset.runs)
-            if USE_SUCCESS_RATE:
-                # Calculate sample efficiency from the success rates of the runs.
-                runset.metric = self.compute_sample_efficiency(runset.runs)
-            else:
-                # Average the run scores (like reward).
-                sum = 0.
-                for run in runset.runs:
-                    sum += run.overall_score
-                    runset.metric = sum / runset.num_runs
+            # Average the run scores (like reward).
+            sum = 0.
+            for run in runset.runs:
+                sum += run.overall_score
+                runset.metric = sum / runset.num_runs
 
     def report(self):
         self.max_runs_per_runset = 0
@@ -323,56 +305,6 @@ class DGD(object):
         # for runset in self.runsets:
         #     runset.report()
 
-    def compute_sample_efficiency(self, runs):
-        global HELDOUT_TESTING
-        if HELDOUT_TESTING:
-            return self.compute_sample_efficiency_by_median(runs)
-        else:
-            return self.compute_sample_efficiency_by_mean_success_rate(runs)
-
-    def compute_sample_efficiency_by_median(self, runs):
-        # SE = median run SE
-        score_sum = 0.
-        num_scores = 0
-        se = 0.
-        ses = np.zeros((len(runs)))
-        for i_run, run in enumerate(runs):
-            for report in run.metric_reports:
-                score_sum += report.score
-                num_scores += 1
-                se = report.sample_efficiency
-            ses[i_run] = se
-        median = np.median(ses)
-        tie_broken_median = median - score_sum / num_scores
-        return tie_broken_median
-
-    def compute_sample_efficiency_by_mean_success_rate(self, runs):
-        # First aggregate the success rates of the runs.
-        
-        num_runs = len(runs)
-        #num_reports = self.max_reports_per_run
-        max_reports = max(len(r.metric_reports) for r in runs)
-
-        score_sum = 0.
-        xs = np.zeros((max_reports))
-        ys = np.zeros((max_reports))
-
-        for run in runs:
-            for i in len(run.metric_reports):
-                score = run.metric_reports[i].score
-                ys[i] += score
-                score_sum += score
-
-        for i in range(max_reports):
-            ys[i] /= num_runs
-            xs[i] = self.runs[0].metric_reports[i].steps
-
-        global SUCCESS_RATE_THRESHOLD
-        se = self.x_at_y(SUCCESS_RATE_THRESHOLD, xs, ys)
-        tie_broken_se = se - score_sum / (num_runs * max_reports)
-
-        return tie_broken_se
-
     def x_at_y(self, y, xs, ys):
         x1 = 0.
         y1 = 0.
@@ -429,15 +361,9 @@ class DGD(object):
         best_runset = self.runsets[0]
         best_metric = best_runset.metric
         for runset in self.runsets:
-            global USE_SUCCESS_RATE
-            if USE_SUCCESS_RATE:
-                if runset.metric <= best_metric:
-                    best_metric = runset.metric
-                    best_runset = runset
-            else:
-                if runset.metric >= best_metric:
-                    best_metric = runset.metric
-                    best_runset = runset
+            if runset.metric >= best_metric:
+                best_metric = runset.metric
+                best_runset = runset
         best_runset.report('Best runset    ')
 
         # Build a neighborhood around (and including) the best runset.
@@ -482,16 +408,16 @@ class DGD(object):
             runset = RunSet(hp_id__setting_id__list, config_str)
         return runset
 
-    def apply_config(self, runset, cf):
-        for hp_i, hparam in enumerate(self.multivalued_hparams):
-            hparam_id = runset.hp_id__setting_id__list[hp_i][0]
-            assert hparam.id == hparam_id
-            
-            value_id = runset.hp_id__setting_id__list[hp_i][1]
-            value = hparam.settings[value_id]
-
-            prop = cf.settings[hparam.name]
-            prop.value = chosen_setting.value
+    # def apply_config(self, runset, cf):
+    #     for hp_i, hparam in enumerate(self.multivalued_hparams):
+    #         hparam_id = runset.hp_id__setting_id__list[hp_i][0]
+    #         assert hparam.id == hparam_id
+    #
+    #         value_id = runset.hp_id__setting_id__list[hp_i][1]
+    #         value = hparam.settings[value_id]
+    #
+    #         prop = cf.settings[hparam.name]
+    #         prop.value = chosen_setting.value
 
     def arg_dict_from_runset(self, runset):
         arg_dict = {}
@@ -516,7 +442,7 @@ class DGD(object):
 
         return arg_dict
 
-if __name__ == '__main__':
+def unit_test():
     dgd = DGD(unit_test=True)
     console.print()
     for hp in dgd.hparams:
@@ -528,4 +454,31 @@ if __name__ == '__main__':
     chosen_runset = dgd.choose_config()
     chosen_runset.report("Chosen runset")
 
+def fake_call_from_job(job_name, workspace, fn_hp_search):
+    from xtlib.helpers.xt_config import get_merged_config
+    config = get_merged_config()
 
+    # build store
+    from xtlib.storage.store import Store
+    store = Store(config=config)
+
+    # build context
+    from xtlib.utils import PropertyBag
+    context = PropertyBag()
+    context.primary_metric = config.get("general", "primary-metric")
+    context.hp_config = fn_hp_search
+    context.search_type = "dgd"
+    context.providers = config.get("providers")
+    context.aggregate_dest = "job"
+    context.dest_name = job_name
+    context.ws = workspace
+
+    from xtlib.hparams.hparam_search import HParamSearch
+    hps = HParamSearch()
+
+    search_file_text = store.read_job_file(context.dest_name, context.hp_config)
+    hps.generate_hparam_set("fn_config", search_file_text, "run999", [], store, context)
+
+if __name__ == '__main__':
+    #unit_test()
+    fake_call_from_job("job363", "tpx", "hp-confg-dir/hp_search.yaml")
