@@ -9,6 +9,7 @@ import sys
 import copy
 import time
 import shutil
+import psutil
 import logging
 
 from xtlib import qfe
@@ -17,9 +18,11 @@ from xtlib.cmd_core import CmdCore
 from xtlib.helpers import xt_config
 from xtlib.impl_base import ImplBase
 from xtlib.helpers import file_helper
-from xtlib.backends.backend_philly import Philly
 from xtlib.helpers.hexdump import hex_dump
 from xtlib.qfe import inner_dispatch, see_also
+from xtlib.backends.backend_philly import Philly
+from xtlib.impl_storage_api import ImplStorageApi
+from xtlib.qfe import inner_dispatch, see_also, keyword_arg
 from xtlib.qfe import command, argument, option, flag, root, example, command_help
 
 from xtlib import utils
@@ -193,7 +196,7 @@ class ImplUtilities(ImplBase):
 
         capture_output = True if cmd else False
 
-        if name.startswith("run"):
+        if run_helper.is_run_name(name):
             # assume it's a RUN name
             rr = run_helper.get_run_record(self.store, workspace, name)
             if not "cluster" in rr:
@@ -275,16 +278,30 @@ class ImplUtilities(ImplBase):
 
         hex_dump(fn)        
 
+    #---- LIST PROCESSES command ----
+    @example(task="xt list processes", text="show the running processes on this box")
+    @command(help="how the name and command line for running processes on the local machine")
+    def list_processes(self):
+
+        processes = psutil.process_iter()
+
+        for p in processes:
+            try:
+                name = p.name().lower()
+                console.print("{}\t{}\t{}".format(p.pid, p.name(), p.cmdline()))
+            except:
+                pass
+
     #---- GROK command ----
-    @option(name="team", default="$general.xt-team-name", help="the team to navigate to on the XT Grok website")
+    @option(name="service-set", default="$service-set", help="the service-set (aka team) to navigate to on the XT Grok website")
     @flag(name="browse", default=False, help="open the XT Grok URL in the browser")
     @example(task="open XT Grok in the browser, for the 'dilbert' team", text="xt browse --team=dilbert")
     @command(help="display or browse the XT Grok website")
-    def grok(self, team, browse):
+    def grok(self, service_set, browse):
 
         url = "https://grok.azurewebsites.net"
         if team:
-            url += "/#/teams/" + team
+            url += "/#/teams/" + service_set
 
         browse = browse and pc_utils.has_gui()
         
@@ -440,13 +457,13 @@ class ImplUtilities(ImplBase):
     #---- CONFIG command ----
     #@flag(name="local", help="edit the config file in the current directory")
     @option(name="response", help="the response to use if a new config file needs to be created")
-    @flag(name="default", help="specifies that the XT DEFAULT config file should be viewed )readonly)")
+    @keyword_arg(name="which", keywords=["factory", "global", "local"], help="specifies which config file should be opened")
     @option(name="create", values=["philly", "batch", "aml", "pool", "all", "empty"], help="specifies that a local XT config file should be created with the specified template")
     @example(task="edit the user's local config file", text="xt config")
     @see_also("XT Config File", "xt_config_file")
     @see_also("Preparing a new project for XT", "prepare_new_project")
     @command(name="config", help="opens an editor on the user's LOCAL XT config file")
-    def config_cmd(self, response, default, create):
+    def config_cmd(self, response, which, create):
         '''
         The --create option accepts a template name to create a new local XT config file.  
         
@@ -459,18 +476,23 @@ class ImplUtilities(ImplBase):
             - empty    (create an empty config file)
         '''
 
-        if default:
+        if which == "factory":
             fn = os.path.join(file_utils.get_xtlib_dir(), "helpers", "default_config.yaml")
             if not os.path.exists(fn):
                 errors.env_error("the XT default config file is missing: {}".format(fn))
+        elif which == "global":
+            fn = os.path.expanduser(os.getenv("XT_GLOBAL_CONFIG", "~/.xt/xt_config.yaml"))
         else:
             fn = constants.CONFIG_FN
 
         edit = True
 
         if create:
+            if which == "factory":
+                errors.general_error("cannot use --create option for the factory config file")
+
             if os.path.exists(fn):
-                console.print("the local config file already exists: {}".format(fn))
+                console.print("the {} XT config file already exists: {}".format(which, fn))
 
                 answer = pc_utils.input_response("OK to overwrite?  (y/n) [n]: ", response)
                 if answer == "y":
@@ -481,7 +503,7 @@ class ImplUtilities(ImplBase):
                 self.create_local_config_file(fn, create)
 
         elif not os.path.exists(fn):
-            console.print("the config file doesn't exist: {}".format(fn))
+            console.print("the {} config file doesn't exist: {}".format(which, fn))
 
             answer = pc_utils.input_response("OK to create?  (y/n) [y]: ", response)
             if answer in ["", "y"]:
@@ -490,6 +512,7 @@ class ImplUtilities(ImplBase):
                 edit = False
 
         if edit:
+            fn = os.path.abspath(fn)       # fix up slashes 
             console.print("invoking your default .yaml editor on: {}".format(fn))
             from xtlib import process_utils 
             process_utils.open_file_with_default_app(fn)
@@ -497,9 +520,10 @@ class ImplUtilities(ImplBase):
     #---- CREATE DEMO command ----
     @argument(name="destination", help="the path in which the demo files should be created")
     @option("response", default=None, help="the response to be used to confirm the directory deletion")
+    @flag("overwrite", help="When specified, any existing xtd-prefixed job names that match xt-demo job names will be overwritten")
     @example(task="create a set of demo files in c:\\xtdemo", text="xt create demo c:/xtdemo")
     @command(help="creates a set of demo files that can be used to quickly try out various XT features")
-    def create_demo(self, destination, response):
+    def create_demo(self, destination, response, overwrite):
         '''
         This command will removed the specified destination directory if it exists (prompting the user for approval).
         Specifying the current directory as the destination will produce an error.
@@ -527,6 +551,14 @@ class ImplUtilities(ImplBase):
             shutil.copytree(from_dir, dest_dir)
             #file_utils.copy_tree(from_dir, dest_dir)
 
+            if not self.store.does_workspace_exist("xt-demo"):
+                # import xt-demo workspace from archive file
+                console.print("importing xt-demo workspace (usually takes about 30 seconds)")
+                impl_storage_api = ImplStorageApi(self.config, self.store)
+
+                fn_archive = os.path.join(file_utils.get_xtlib_dir(), "demo_files", "xt-demo-archive.zip")
+                impl_storage_api.import_workspace(fn_archive, "xt-demo", "xtd", overwrite=overwrite, show_output=False)
+
     def is_aml_ws(self, ws_name):
         return self.azure_ml.does_ws_exist(ws_name)
         
@@ -535,7 +567,7 @@ class ImplUtilities(ImplBase):
 
         if not env:
             compute_def = self.config.get_compute_def(compute)
-            env = utils.safe_value(compute_def, "environment")
+            env = utils.safe_value(compute_def, "docker")
 
         if env and env != "none":
             env_def = self.config.get("dockers", env, default_value=None)
@@ -553,16 +585,21 @@ class ImplUtilities(ImplBase):
 
     #---- DOCKER LOGIN command ----
     @option("docker", help="the docker environment that defines the docker registry for login")
-    @option("target", default="$xt-services.target", help="one of the user-defined compute targets on which to run")
+    @option("target", default=None, type=str, help="one of the user-defined compute targets on which to run")
     @example(task="log user into docker", text="xt docker login")
     @command(kwgroup="docker", kwhelp="runs the specified docker command", help="logs the user into docker using docker credentials from the XT config file")
-    def docker_login(self, target, environment):
-        reg_creds = self.get_registry_creds(target, environment)
+    def docker_login(self, target, docker):
+
+        if not target:
+            ss_info = self.get_service_set_info()
+            target = utils.safe_value(ss_info, "target")
+
+        reg_creds = self.get_registry_creds(target, docker)
         if not reg_creds:
-            if environment:
-                errors.env_error("no docker environment defined for environment '{}'".format(environment))
+            if docker:
+                errors.env_error("no dockers entry defined for docker '{}'".format(docker))
             else:
-                errors.env_error("no docker environment defined for target '{}'".format(target))
+                errors.env_error("no docker property defined for target '{}'".format(target))
 
         server = reg_creds["login-server"]
         username = reg_creds["username"]
@@ -573,11 +610,16 @@ class ImplUtilities(ImplBase):
 
     #---- DOCKER LOGOUT command ----
     @option("docker", help="the docker environment that defines the docker registry for login")
-    @option("target", default="$xt-services.target", help="one of the user-defined compute targets on which to run")
+    @option("target", default=None, type=str, help="one of the user-defined compute targets on which to run")
     @example(task="log the user out of docker", text="xt docker logout")
     @command(kwgroup="docker", help="logs the user out of docker")
-    def docker_logout(self, target, environment):
-        reg_creds = self.get_registry_creds(target, environment)
+    def docker_logout(self, target, docker):
+
+        if not target:
+            ss_info = self.get_service_set_info()
+            target = utils.safe_value(ss_info, "target")
+
+        reg_creds = self.get_registry_creds(target, docker)
         server = reg_creds["login-server"]
 
         text = self.core.docker_logout(server)
