@@ -25,7 +25,8 @@ class PlotBuilder():
     def __init__(self, run_names, col_names, x_col, layout, break_on, title, show_legend, plot_titles,
             legend_titles, smoothing_factor, plot_type, timeout,
             aggregate, shadow_type, shadow_alpha, run_log_records, style, show_toolbar, max_runs, max_traces,
-            group_by, error_bars, show_plot, save_to, x_label, colors, color_map, color_steps, legend_args, plot_args):
+            group_by, error_bars, show_plot, save_to, x_label, colors, color_map, color_steps, legend_args, 
+            x_min, x_max, y_min, y_max, alpha, line_size, timebase, x_format, y_format, plot_args):
         
         self.run_names = run_names
         self.col_names = col_names
@@ -54,6 +55,15 @@ class PlotBuilder():
         self.x_label = x_label
         self.legend_args = legend_args
         self.plot_args = plot_args
+        self.x_min = x_min
+        self.x_max = x_max
+        self.y_min = y_min
+        self.y_max = y_max
+        self.alpha = alpha
+        self.line_size = line_size
+        self.timebase = timebase
+        self.x_format = x_format
+        self.y_format = y_format
 
         if colors:
             self.colors = colors
@@ -102,39 +112,59 @@ class PlotBuilder():
         pp_run_names = []
         used_max = False
         data_frames_by_cols = {}
-        got_columns = False
+        explicit_options = qfe.get_explicit_options()
+        missing_col_runs = 0
 
         for i, record in enumerate(self.run_log_records):
             # extract metrics for this run
-            run = record["_id"]
+            run = record["run_name"]
             node = utils.node_id(record["node_index"])
             job = record["job_id"]
             experiment = record["exper_name"]
-            workspace = record["ws"]
+            workspace = record["ws_name"]
+            search_style = utils.safe_value(record, "search_style")
+            if search_style and search_style != "single":
+                # parent run with children - skip it
+                continue
 
             log_records = record["log_records"]
 
-            metric_sets = run_helper.build_metrics_sets(log_records)
+            metric_sets = run_helper.build_metrics_sets(log_records, timebase=self.timebase)
             if not metric_sets:
                 no_metrics.append(run)
-                continue
+                continue    # SKIP this run
 
             if self.max_runs and len(pp_run_names) >= self.max_runs:
                 used_max = True
                 break
 
-            if not got_columns:
-                # set x and y columns
-                explicit = qfe.get_explicit_options()
-                if not "x" in explicit:
-                    self.x_col = self.get_actual_x_column(metric_sets, self.x_col, self.col_names)
-
-                if not self.col_names:
-                    # not specified by user, so build defaults
-                    self.col_names = self.get_default_y_columns(metric_sets, self.x_col)
-
-                got_columns = True
+            if self.x_col:
+                # ensure x_col is in this run
+                if not self.is_col_in_sets(metric_sets, self.x_col):
+                    console.print("skipping: {} (missing metric: {})".format(run, self.x_col))
+                    missing_col_runs += 1
+                    continue    # SKIP this run
+            else:
+                #if not "x" in explicit_options:
+                self.x_col = self.get_actual_x_column(metric_sets, self.x_col, self.col_names)
                 
+            if self.col_names:
+                # ensure col_names are in run
+                found_all = True
+
+                for col in self.col_names:
+                    if not self.is_col_in_sets(metric_sets, col):
+                        console.print("skipping: {} (missing metric: {})".format(run, col))
+                        found_all = False
+                        break
+
+                if not found_all:
+                    missing_col_runs += 1
+                    continue    # SKIP this run
+            else:
+                # not specified by user, so build defaults
+                self.col_names = self.get_default_y_columns(metric_sets, self.x_col)
+
             # merge metric sets into dfx
             for metric_set in metric_sets:
 
@@ -177,8 +207,20 @@ class PlotBuilder():
 
         if used_max:
             console.print("plotting first {} runs (use --max-runs to override)".format(self.max_runs))
-        else:
+        elif pp_run_names:
             console.print("plotting {} runs...".format(len(pp_run_names)))
+        else:
+            console.print("no matching runs found")
+            data_frames_by_cols = None
+
+            if missing_col_runs:
+                # some runs missing columns - display list of columns in all metric sets
+                cols = set()
+                for ms in metric_sets:
+                    cols = cols.union(ms["keys"])
+
+                cols = ", ".join(list(cols))
+                console.print("available metrics to plot: \n  {}".format(cols))
 
         # update our list of run_names to proces
         self.run_names = pp_run_names
@@ -226,7 +268,7 @@ class PlotBuilder():
                 group_prefix = "node" if self.group_by == "node_index" else ""
                 x_col = self.x_col
 
-                dfx = dfx.groupby([group_col, x_col])
+                dfx = dfx.groupby([group_col, x_col], sort=False)
             
             # AGGREGATE data
             df_agg_from = dfx
@@ -327,7 +369,10 @@ class PlotBuilder():
                     y_values = df[ y_col ].to_numpy(dtype=float)
 
                     if x_col and x_col in df.columns:
-                        x_values = df[ x_col ].to_numpy(dtype=float)
+                        if x_col == "__time__":
+                             x_values = df[ x_col ].to_numpy()
+                        else:
+                            x_values = df[ x_col ].to_numpy(dtype=float)
 
                     if stat_col and stat_col in df.columns:
                         stat_values = df[ stat_col ].to_numpy(dtype=float)
@@ -352,22 +397,26 @@ class PlotBuilder():
         if self.style:
             sns.set_style(self.style)
 
-        # decide how layout, titles, etc. will be set
-        group_names = set()
-
         # gather group names (usually all are in the first dataset, but not always)
+        # preserve the order names were found in (don't use a set)
+        group_names = {}     # python 3.6+ preserves name order in dicts
+
         for dfx in data_frames_by_cols.values():
             name_list = dfx[self.group_by].unique()
-            group_names.update(set(name_list))
+
+            for name in name_list:
+                if not name in group_names:
+                    group_names[name] = 1
         
         group_names = list(group_names)
 
         # this will sort the group names in a number-smart way 
-        group_names.sort(key=utils.natural_keys)
+        #group_names.sort(key=utils.natural_keys)
 
         group_count = len(group_names)
         col_count = len(self.col_names)
 
+        # decide how layout, titles, etc. will be set
         break_on_groups = self.break_on and ("run" in self.break_on or "group" in self.break_on)
         break_on_cols = self.break_on and "col" in self.break_on
 
@@ -412,6 +461,7 @@ class PlotBuilder():
 
         #plt.close()
         window_size = (14, 6)
+        #window_size = (20, 12)
 
         fig, plots = plt.subplots(plot_rows, plot_cols, figsize=window_size, sharex=sharex, sharey=sharey, constrained_layout=True)
         if not isinstance(plots, np.ndarray):
@@ -420,6 +470,17 @@ class PlotBuilder():
         elif plot_rows > 1:
             plots = plots.flatten()
 
+        # set axis limit
+        if self.x_min:
+            plt.xlim(left=self.x_min)
+        if self.x_max:
+            plt.xlim(right=self.x_max)
+
+        if self.y_min:
+            plt.ylim(bottom=self.y_min)
+        if self.y_max:
+            plt.ylim(top=self.y_max)
+
         fig.suptitle(self.title, fontsize=16)
 
         if self.timeout:
@@ -469,7 +530,7 @@ class PlotBuilder():
 
                     # PLOT MIDDLE
                     ax = plots[plot_index] # .gca()
-                    self.plot_middle(data_frames_by_cols, ax, group_name, col, self.x_col, x_label, line_index)
+                    self.plot_middle(data_frames_by_cols, ax, group_name, col, self.x_col, x_label, line_index, plot_index)
                     line_index += 1
                     trace_count += 1
         else:
@@ -494,13 +555,40 @@ class PlotBuilder():
 
                     # PLOT MIDDLE
                     ax = plots[plot_index] #.gca()
-                    self.plot_middle(data_frames_by_cols, ax, group_name, col, self.x_col, x_label, line_index)
+                    self.plot_middle(data_frames_by_cols, ax, group_name, col, self.x_col, x_label, line_index, plot_index)
                     line_index += 1
                     trace_count += 1
 
         if self.save_to:
             plt.savefig(self.save_to)
 
+        # maximize plot window
+        # mgr = plt.get_current_fig_manager()
+        # mgr.window.full_screen_toggle(True)
+
+        ax = plots[0]
+
+        if self.x_col == "__time__":
+            import matplotlib.dates as mdates
+
+            locator = mdates.AutoDateLocator(minticks=10, maxticks=20)
+            formatter = mdates.ConciseDateFormatter(locator)
+
+            ax.xaxis.set_major_locator(locator)
+            ax.xaxis.set_major_formatter(formatter)            
+            #plt.gcf().autofmt_xdate()
+        elif self.x_format:
+            import matplotlib
+            user_fmt = lambda x, p: format(x, self.x_format)
+            ff = matplotlib.ticker.FuncFormatter(user_fmt)
+            ax.get_xaxis().set_major_formatter(ff)
+
+        if self.y_format:
+            import matplotlib
+            user_fmt = lambda y, p: format(y, self.y_format)
+            ff = matplotlib.ticker.FuncFormatter(user_fmt)
+            ax.get_yaxis().set_major_formatter(ff)
+
         if self.show_plot:
             pylab.show()
 
@@ -516,7 +604,7 @@ class PlotBuilder():
         cmap = ListedColormap(sns.color_palette(current_palette).as_hex())
         return cmap
 
-    def plot_middle(self, data_frames_by_cols, ax, group_name, col, x_col, x_label, line_index):
+    def plot_middle(self, data_frames_by_cols, ax, group_name, col, x_col, x_label, line_index, plot_index):
         
         color_index = line_index % len(self.colors)
         color = self.colors[color_index]
@@ -525,7 +613,7 @@ class PlotBuilder():
             # draw PRESMOOTH SHADOW
             x, y, _ = self.get_xy_values(data_frames_by_cols, group_name, self.x_col, col + PRESMOOTH, None)
 
-            self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, x_values=x, y_values=y,
+            self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, plot_index, x_values=x, y_values=y,
                 color=color, alpha=self.shadow_alpha, use_y_label=False)
         elif self.shadow_type:
 
@@ -541,17 +629,17 @@ class PlotBuilder():
                 y = y_mean - y_stat
                 y2 = y_mean + y_stat
 
-            self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, x_values=x, y_values=y, 
+            self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, plot_index, x_values=x, y_values=y, 
                 color=color, alpha=self.shadow_alpha, use_y_label=False, y2_values=y2)
 
         # DRAW NORMAL LINE
         err_col = col + "_{}_".format(self.error_bars.upper()) if self.error_bars else None
         x, y, err = self.get_xy_values(data_frames_by_cols, group_name, self.x_col, col, err_col)
 
-        self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, x_values=x, y_values=y, 
-            color=color, alpha=1, use_y_label=True, err_values=err)
+        self.plot_inner(ax, group_name, col, self.x_col, x_label, line_index, plot_index, x_values=x, y_values=y, 
+            color=color, alpha=self.alpha, use_y_label=True, err_values=err)
 
-    def plot_inner(self, ax, run_name, col, x_col, x_label, line_index, x_values, y_values, 
+    def plot_inner(self, ax, group_name, col, x_col, x_label, line_index, plot_index, x_values, y_values, 
         color, alpha, use_y_label, y2_values=None, err_values=None):
 
         import seaborn as sns
@@ -572,7 +660,7 @@ class PlotBuilder():
 
         if use_y_label:
             line_title = self.legend_titles[line_index % len(self.legend_titles)]
-            line_title = self.fixup_text(line_title, run_name, col)
+            line_title = self.fixup_text(line_title, group_name, col)
         else:
             line_title = None
 
@@ -580,7 +668,7 @@ class PlotBuilder():
         is_range_plot = bool(y2_values is not None)
 
         # our default attributes
-        kwargs = {"label": line_title, "color": color, "alpha": alpha}
+        kwargs = {"label": line_title, "color": color, "alpha": alpha, "linewidth": self.line_size}
         
         if not is_range_plot:
             kwargs["capsize"] = cap_size
@@ -615,8 +703,8 @@ class PlotBuilder():
             errors.syntax_error("unknown plot type={}".format(self.plot_type))
 
         if self.plot_titles:
-            plot_title = self.plot_titles[line_index % len(self.plot_titles)]
-            plot_title = self.fixup_text(plot_title, run_name, col)
+            plot_title = self.plot_titles[plot_index % len(self.plot_titles)]
+            plot_title = self.fixup_text(plot_title, group_name, col)
 
             ax.set_title(plot_title)
 
@@ -627,11 +715,23 @@ class PlotBuilder():
                 # pass legend args to legend object
                 ax.legend(**self.legend_args)
 
-    def fixup_text(self, text, run_name, col):
-        text = text.replace("$run", run_name)
+    def fixup_text(self, text, group_name, col):
+        text = text.replace("$job", group_name)
+        text = text.replace("$run", group_name)
         text = text.replace("$col", col)
         return text
 
+    def is_col_in_sets(self, metric_sets, col_name):
+        found = False
+
+        for ms in metric_sets:
+            keys = ms["keys"]
+            if col_name in keys:
+                found = True
+                break
+
+        return found
+
     def get_actual_x_column(self, metric_sets, default_x_col, y_cols):
         '''
         x col search order:
@@ -683,37 +783,3 @@ class PlotBuilder():
 
         return y_cols
 
-    def range_runs(self, runs_dict, range):
-        runs = list(runs_dict.values())
-
-        if range == "min-max":
-            min_values = np.min(runs, axis=0)
-            max_values = np.max(runs, axis=0)
-        elif range == "std":
-            means = np.mean(runs, axis=0)
-            max_values = means + np.std(runs, axis=0)
-            min_values = means - np.std(runs, axis=0)
-        elif range == "error":
-            from scipy import stats
-
-            means = np.mean(runs, axis=0)
-            max_values = means + stats.sem(runs, axis=0)
-            min_values = means - stats.sem(runs, axis=0)
-        else:
-            errors.syntax_error("unrecognized range value: {}".format(range))
-
-        return min_values, max_values
-
-    def get_values_by_run(self, col, run_log_records):
-        runs_dict = {}
-
-        for rr in run_log_records:
-            run_name = rr["_id"]
-            value_recs = rr["metrics"]["records"]
-            new_values = [vr[col] for vr in value_recs]
-
-            runs_dict[run_name] = new_values
-
-        return runs_dict
-
-    
\ No newline at end of file
