@@ -9,6 +9,7 @@ import time
 import arrow
 import datetime
 import logging
+import numpy as np
 from fnmatch import fnmatch
 from collections import OrderedDict
 
@@ -154,7 +155,7 @@ class ReportBuilder():
 
         std_list.sort()
 
-        lines.append("")
+        #lines.append("")
         lines.append("Standard {} columns:".format(report_type))
         for col in std_list:
             if col in ["hparams", "metrics", "tags"]:
@@ -285,7 +286,10 @@ class ReportBuilder():
     def get_mongo_records(self, mongo, filter_dict, workspace, which, actual_to_user, 
             col_dict=None, args=None):
 
+        started = time.time()
+
         first, last = self.get_first_last(args)
+        skip = utils.safe_value(args, "skip")
 
         if last:
             using_default_last = True
@@ -296,47 +300,56 @@ class ReportBuilder():
         # use MONGO to do all of the work (query, sort, first/last)
         sort_col = utils.safe_value(args, "sort", "name")
 
-        if sort_col == "name":
-            # special sorting needed; we have created "run_num" field just for this purpose
-            sort_col = "run_num" if which == "runs" else "job_num"
-        elif not "." in sort_col:
-            # translate name of std col from user-friendly version to logged version
-            user_to_actual = {value: key for key, value in actual_to_user.items()}
+        if sort_col:
+            if sort_col == "name":
+                # special sorting needed; we have created "run_num" field just for this purpose
+                sort_col = "run_num" if which == "runs" else "job_num"
+            elif not "." in sort_col:
+                # translate name of std col from user-friendly version to logged version
+                user_to_actual = {value: key for key, value in actual_to_user.items()}
 
-            if not sort_col in user_to_actual:
-                errors.general_error("unknown standard property: {} (did you mean metrics.{}, hparams.{}, or tags.{}?)". \
-                    format(sort_col, sort_col, sort_col, sort_col))
+                if not sort_col in user_to_actual:
+                    errors.general_error("unknown standard property: {} (did you mean metrics.{}, hparams.{}, or tags.{}?)". \
+                        format(sort_col, sort_col, sort_col, sort_col))
 
-            sort_col = user_to_actual[sort_col]
+                sort_col = user_to_actual[sort_col]
 
-        # this is a TRICK to avoid having to call for the exists_count for calculation of skip count
-        # it works fine, since we re-sort records on the xt client anyway
-        sort_dir = -1 if reverse else 1
-        if last:
-            sort_dir = -sort_dir
-            first = last
+            # this is a TRICK to avoid having to call for the exists_count for calculation of skip count
+            # it works fine, since we re-sort records on the xt client anyway
+            sort_dir = -1 if reverse else 1
+            if last:
+                sort_dir = -sort_dir
+                first = last
 
-        # ensure we only ask for records where sort_col exists, or else we MIGHT end up with less than LIMIT records
-        if not sort_dir in filter_dict:
-            filter_dict[sort_col] = { "$exists": True}
+            # ensure we only ask for records where sort_col exists, or else we MIGHT end up with less than LIMIT records
+            if sort_col != "run_num" and not sort_col in filter_dict:
+                filter_dict[sort_col] = { "$exists": True}
 
-        container = workspace if which == "runs" else "__jobs__"
+        # always include ws_name (for now)
+        if workspace and not "ws_name" in filter_dict:
+            filter_dict["ws_name"] = workspace
+
+        container = "run_info" if which == "runs" else "job_info"
 
         orig_col_dict =  col_dict
-        if not col_dict:
-            col_dict = {"log_records": 0}
+        # if not col_dict:
+        #     col_dict = {"log_records": 0}
 
-        # put our mongo operations together in a retry-compatible function
-        def fetch():
-            cursor = mongo.mongo_db[container].find(filter_dict, col_dict)
-            cursor = cursor.sort(sort_col, 1 if not last else -1)
-            if first:
-                cursor = cursor.limit(first)
-            return cursor
+        console.diag("get_mongo_records: first={}, last={}, ws: {}, filter_dict: {}, col_dict: {}". \
+            format(first, last, workspace, filter_dict, col_dict))
+
+        count_runs = utils.safe_value(args, "count")
+        buffer_size = utils.safe_value(args, "buffer_size", 50)
 
         # here is where MONGO does all the hard work for us
-        cursor = mongo.mongo_with_retries("get_mongo_records", fetch)
-        records = list(cursor)
+        if which == "runs":
+            records = mongo.get_filtered_sorted_run_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
+                count_runs, buffer_size)
+        else:
+            records = mongo.get_filtered_sorted_job_info(workspace, filter_dict, col_dict, sort_col, sort_dir, skip, first, 
+                count_runs, buffer_size)
+
+        round_trip_count = mongo.round_trip_count
 
         console.diag("after full records retreival, len(records)={}".format(len(records)))
 
@@ -352,6 +365,9 @@ class ReportBuilder():
             records.reverse()
             #self.sort_records(records, sort_col, reverse)
 
+        elapsed = time.time() - started
+        #console.print("  query stats: round trips={}, elapsed: {:.2f} secs".format(round_trip_count, elapsed))
+
         return records, using_default_last, last
 
     def get_user_columns(self, args):
@@ -391,6 +407,7 @@ class ReportBuilder():
         '''
         row_count = 0
         was_exported = False
+        max_column_width = utils.safe_value(args, "max_width")
 
         avail_list, user_col_args  = self.get_actual_and_user_cols(records, args)
 
@@ -414,7 +431,9 @@ class ReportBuilder():
             actual_cols = list(user_col_args.keys())
 
             text, row_count = self.build_formatted_table(records, avail_cols=avail_list, col_list=actual_cols, 
-                report_type=report_type, group_by=group_by, number_groups=number_groups)
+                report_type=report_type, group_by=group_by, number_groups=number_groups, 
+                max_col_width=max_column_width)
+
             lines = text.split("\n")
 
         return lines, row_count, was_exported
@@ -486,21 +505,32 @@ class ReportBuilder():
         blank_zero_fmt = "{:$bz}"
         date_only = "{:$do}"
         time_only = "{:$to}"
+        right_trim = "{:$rt."
 
         if fmt == blank_zero_fmt:
             # blank if zero
             value = "" if value == 0 else str(value)
         elif fmt == date_only:
             # extract date portion of datetime string
-            value, _ = value.split(" ")
+            delim = " " if " " in value else "T"
+            value, _ = value.split(delim)
         elif fmt == time_only:
             # extract time portion of datetime string
-            _, value = value.split(" ")
+            delim = " " if " " in value else "T"
+            _, value = value.split(delim)
+            value = value.split(".")[0]    # extract hh:mm:ss part of fractional time
+        elif fmt.startswith(right_trim):
+            int_part = fmt[6:].split("}")[0]
+            trim_len = int(int_part)
+            value = str(value)[-trim_len:]
         
         return value
 
     def needed_precision(self, value, significance):
         # start with scientific notation formatting
+        if value != value:   # test for nan
+            return 0
+
         text = "%.*e" % (significance-1, value)
 
         # how much precision is required for specified significance?
@@ -533,7 +563,7 @@ class ReportBuilder():
         'actual_cols' - list of columns to be used for report (strict subset of 'avail_cols')
         '''
 
-        time_col_names = ["created", "started", "ended"]
+        time_col_names = ["created", "started", "ended", "time"]
         duration_col_names = ["duration", "queued"]
 
         #console.print("self.user_col_args=", self.user_col_args)
@@ -564,8 +594,8 @@ class ReportBuilder():
         # build COL_INFO for each col (will calcuate col WIDTH, formatting, etc.)
         for i, col in enumerate(col_list):
 
-            if col == "hparams.lr":
-                print()
+            # if col == "hparams.lr":
+            #     print()
             # largest precision needed for this col
             max_precision = precision 
 
@@ -801,12 +831,11 @@ class ReportBuilder():
                         int_fmt = "{:" + align + str(col_width) + "d}"
                         text += int_fmt.format(value)
                     else:
+                        if col == "sku":
+                            dummy = 3
                         # ensure value is a string
-                        value = str(value)
-                        
+                        value = "" if value is None else str(value)
                         str_fmt = "{:" + align + str(col_width)  + "." + str(col_width) + "s}"
-                        #value = value if value else ""
-                        value = "" if value is None else value
                         text += str_fmt.format(value)
             text += "\n"
             row_count += 1
