@@ -15,11 +15,13 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
+from xtlib import store_utils
 from xtlib import impl_storage
 
 from xtlib.storage.store import Store, store_from_context
 from xtlib.console import console
 from xtlib.helpers.xt_config import get_merged_config
+from xtlib.helpers.key_press_checker import KeyPressChecker
 
 FN_CHECKPOINT_FILE = "checkpoints/file.dat"
 FN_CHECKPOINT_DICT = "checkpoints/dict_cp.json"
@@ -46,6 +48,7 @@ class Run():
         self.metric_report_count = 0
         self.metric_names = OrderedDict()
         self.supress_normal_output = supress_normal_output
+        self.user_requests = []
 
         # tensorboard writers
         self.train_writer = None
@@ -73,6 +76,7 @@ class Run():
         self.ws_name = os.getenv("XT_WORKSPACE_NAME", None)
         self.exper_name = os.getenv("XT_EXPERIMENT_NAME", None)
         self.run_name = os.getenv("XT_RUN_NAME", None)
+        self.job_id = os.getenv("XT_JOB_ID", None)
         self.resume_name = os.getenv("XT_RESUME_NAME")
 
         # load context, if present
@@ -152,13 +156,42 @@ class Run():
 
         self.direct_run = not os.getenv("XT_CONTROLLER")
 
-        if  self.xt_logging and self.direct_run and self.store:
+        if self.xt_logging and self.direct_run and self.store:
             # log stuff normally done by controller at start of run
             self.store.log_run_event(self.ws_name, self.run_name, "started", {})   
-            self.store.mongo.run_start(self.ws_name, self.run_name)
-            if self.context:
-                self.store.mongo.job_run_start(self.context.job_id)
 
+            if self.store.mongo:
+                self.store.mongo.run_start(self.ws_name, self.run_name)
+                if self.context:
+                    self.store.mongo.job_run_start(self.ws_name, self.context.job_id)
+
+    def check_for_user_request(self):
+
+        request = None
+
+        if self.run_name:
+            # running under XT
+            # check user request file (filled by controller)
+            fn = os.path.expanduser(constants.FN_USER_REQUEST)
+            if os.path.exists(fn):
+                # TODO: make read/delete an ATOMIC operation
+                text = file_utils.read_text_file(fn)
+                file_utils.zap_file(fn)
+
+                requests = text.split("\n")
+                self.user_requests += requests
+
+            if self.user_requests:
+                request_text = self.user_requests.pop(0)
+                if request_text:
+                    request = json.loads(request_text)
+        else:
+            # if running standalone, just see if a keypress event is available
+            with KeyPressChecker() as checker:
+                request = checker.getch_nowait()
+
+        return request
+        
     def init_tensorboard(self):
         # as of Oct-04-2019, to use torch.utils.tensorboard on DSVM systems, we need to do one of the following:
         #   - clear the env var PYTHONPATH (before running this app)
@@ -221,6 +254,10 @@ class Run():
             self.train_writer2.close()
             self.test_writer2.close()
 
+        if os.getenv("XT_MOUNTING_ENABLED") == "False":
+            console.print("run.close(): uploading output directory to RUN storage")
+            self.upload_output_dir()
+
         if self.is_aml and self.store:
             # partially log the end of the run
 
@@ -308,7 +345,8 @@ class Run():
                 # if not self.supress_normal_output:
                 #     console.print("updating metric_names: {}".format(ddx))
 
-                self.store.mongo.update_mongo_run_from_dict(self.ws_name, self.run_name, ddx)
+                if self.store.mongo:
+                    self.store.mongo.update_run_info(self.ws_name, self.run_name, ddx)
 
         if self.is_aml and self.aml_logging:
             for name, value in dd.items():
@@ -389,7 +427,19 @@ class Run():
 
         return dict_cp
 
-    def upload_files_to_share(self, share, store_path, local_path, show_feedback):
+    def upload_output_dir(self, local_path=None, show_feedback=False):
+
+        if not local_path:
+            local_path = os.getenv("XT_OUTPUT_DIR", "output")
+
+        #store_path = "$jobs/{}/runs/run/output".format(self.job_id, self.run_name)
+        store_path = "$" + store_utils.get_run_path(self.job_id, self.run_name) + "/output"
+        storage = impl_storage.ImplStorage(self.config, self.store)
+
+        count = storage.upload(local_path=local_path, store_path=store_path, share=None, feedback=show_feedback, 
+            workspace=self.ws_name, run=None, experiment=None, job=None)
+
+    def upload_files_to_share(self, share, store_path, local_path, show_feedback=False):
         '''
         note: show_feedback not implemented; caller should call console.set_level() to control type of output
         emitted by XTLib API.
@@ -399,18 +449,16 @@ class Run():
         count = storage.upload(local_path=local_path, store_path=store_path, share=share, feedback=show_feedback, 
             workspace=None, run=None, experiment=None, job=None)
 
-        path = "store://{}/{}".format(utils.make_share_name(share), store_path)
-
-
+        path = "store://{}/{}".format(store_utils.make_share_name(share), store_path)
         return {"count": count, "path": path}
 
-    def download_files_from_share(self, share, store_path, local_path, show_feedback, snapshot=True):
+    def download_files_from_share(self, share, store_path, local_path, show_feedback=False, snapshot=False):
         storage = impl_storage.ImplStorage(self.config, self.store)
         
         count = storage.download(local_path=local_path, store_path=store_path, share=share, 
             feedback=show_feedback, snapshot=snapshot, workspace=None, run=None, experiment=None, job=None)
             
-        path = "store://{}/{}".format(utils.make_share_name(share), store_path)
+        path = "store://{}/{}".format(store_utils.make_share_name(share), store_path)
         return {"count": count, "path": path}
 
     def get_next_hp_set_in_search(self, hp_space_dict, search_type=None, hparam_search=None):
