@@ -23,6 +23,8 @@ from xtlib import utils
 from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
+from xtlib import store_utils
+from xtlib import run_helper
 
 from xtlib.console import console
 
@@ -46,6 +48,10 @@ class StoreBlobObjs():
         self.max_retries = max_retries
         self.set_retries(max_retries)
 
+    def get_name(self):
+        storage_name = self.provider.get_service_name()
+        return storage_name
+        
     def validate_storage_and_mongo(self, mongo):
         '''
         1. ensure storage has been initialized for XT
@@ -73,18 +79,24 @@ class StoreBlobObjs():
             errors.combo_error("this storage paired with mongo service='{}', but passed connection string for mongo service='{}'".format(  \
                 connected_mongo, paired_mongo))
 
+        if storage_info["storage_format"] != constants.STORAGE_FORMAT:
+            errors.store_error("XT storage format {} expected but found: {}".format(constants.STORAGE_FORMAT, storage_info["storage_format"]))
+
         if not paired_storage:
-            mongo_info = {"paired_storage": storage_name, "storage_version": constants.STORAGE_VERSION}
+            mongo_info = {"paired_storage": storage_name, "storage_format": constants.STORAGE_FORMAT}
             mongo.set_mongo_info(mongo_info)
 
         if not paired_mongo:
-            storage_info = {"paired_mongo": connected_mongo, "storage_version": constants.STORAGE_VERSION}
+            storage_info = {"paired_mongo": connected_mongo, "storage_format": constants.STORAGE_FORMAT}
             self._set_storage_info(storage_info)
 
+        if mongo_info["storage_format"] != constants.STORAGE_FORMAT:
+            errors.store_error("XT mongo format {} expected but found: {}".format(constants.STORAGE_FORMAT, mongo_info["storage_format"]))
+
         # only check once, (takes .5 secs if already imported)
         # remove this check after all XT users have imported (approx. Dec 2019)
         # but keep around (good for mongodb repair, if needed)
-        self.import_jobs_to_mongo_if_needed(mongo)
+        #self.import_jobs_to_mongo_if_needed(mongo)
 
     def import_jobs_to_mongo_if_needed(self, mongo):
         console.diag("before mongo import check")
@@ -101,7 +113,8 @@ class StoreBlobObjs():
                 for job_id in job_names:
                     job_json = self.read_job_info_file(job_id)
                     dd = json.loads(job_json)
-                    mongo.update_job_info(job_id, dd)
+                    ws_name = dd["ws"]
+                    mongo.update_job_info(ws_name, job_id, dd)
                     count += 1
                     if count % 100 == 0:
                         console.print("  " + job_id)
@@ -129,17 +142,17 @@ class StoreBlobObjs():
                 self.provider.create_container(info_container)
 
             # create storage info file
-            info = {"version": constants.STORAGE_VERSION, "paired_mongo": None}
+            info = {"storage_format": constants.STORAGE_FORMAT, "paired_mongo": None}
             text = json.dumps(info)
             self._create_blob(info_container, constants.STORAGE_INFO_FILE, text)
 
             # create the DATA SHARE if needed
-            if not self.provider.does_container_exist(utils.DATA_STORE_ROOT):
-                self.provider.create_container(utils.DATA_STORE_ROOT)
+            if not self.provider.does_container_exist(store_utils.DATA_STORE_ROOT):
+                self.provider.create_container(store_utils.DATA_STORE_ROOT)
 
             # create the MODELS SHARE if needed
-            if not self.provider.does_container_exist(utils.MODELS_STORE_ROOT):
-                self.provider.create_container(utils.MODELS_STORE_ROOT)
+            if not self.provider.does_container_exist(store_utils.MODELS_STORE_ROOT):
+                self.provider.create_container(store_utils.MODELS_STORE_ROOT)
 
     # ---- INTERNAL HELPERS ----
     def _check_ws_name(self, ws_name):
@@ -163,10 +176,13 @@ class StoreBlobObjs():
         return constants.EXPERIMENTS_DIR + "/" + exper_name + "/" + fn
 
     def _run_path(self, run_name):
-        return constants.RUNS_DIR + "/" + run_name 
+        parent_num = run_helper.get_parent_run_number(run_name)
+        path = "jobs/job{}/runs/{}".format(parent_num, run_name)
+        return path
 
     def _make_run_path_fn(self, run_name, fn):
-        return constants.RUNS_DIR + "/" + run_name + "/" + fn
+        path = self._run_path(run_name)
+        return path + "/" + fn
 
     def _remove_first_node(self, path):
         ''' remove first node of path '''
@@ -261,7 +277,7 @@ class StoreBlobObjs():
 
         # ensure the container exists
         if not self.does_workspace_exist(ws_name):
-            self.create_workspace(ws_name, description=None)
+            self.create_workspace(ws_name)
 
         if source_wildcard.endswith("**"):
             # handle special "**" for recursive copy
@@ -362,6 +378,11 @@ class StoreBlobObjs():
         console.diag("_list_directories: container={}, path={}, wc_target={}, subdirs={}".format(
             container, path, wc_target, subdirs))
 
+        # special case: if container/path exists as a blob, treat as if wildcard on parent
+        if not wc_target and container and self.provider.does_blob_exist(container, path):
+            wc_target = os.path.basename(path)
+            path = os.path.dirname(path)
+
         service_name = self.provider.get_service_name()
         dd = {"store_name": "XT Store ({})".format(service_name)}
         #console.print("dd=", dd)
@@ -394,7 +415,7 @@ class StoreBlobObjs():
 
             if wc_target:
                 # apply filter
-                blobs = [blob for blob in blobs if fnmatch(blob.name, wc_target)]
+                blobs = [blob for blob in blobs if fnmatch(os.path.basename(blob.name), wc_target)]
 
             console.diag("list_blobs returned: len(blobs)={}".format(len(blobs)))
 
@@ -551,13 +572,13 @@ class StoreBlobObjs():
     # ---- SHARES ----
 
     def does_share_exist(self, share_name):
-        container_name = utils.make_share_name(share_name)
+        container_name = store_utils.make_share_name(share_name)
         #console.print("does_share_exist: ws_name=", ws_name)
         self._check_ws_name(container_name)
         return self.provider.does_container_exist(container_name)
 
     def ensure_share_exists(self, share_name, flag_as__error=True):
-        container_name = utils.make_share_name(share_name)
+        container_name = store_utils.make_share_name(share_name)
         self._check_ws_name(container_name)
         exists = self.does_share_exist(share_name)
         if not exists:
@@ -568,7 +589,7 @@ class StoreBlobObjs():
     def create_share(self, share_name, description=None):
         ''' create share as top level container '''
 
-        container_name = utils.make_share_name(share_name)
+        container_name = store_utils.make_share_name(share_name)
         self._check_ws_name(container_name)
 
         # note: this operation often must retry several times if same container has just been deleted
@@ -580,7 +601,7 @@ class StoreBlobObjs():
              errors.store_error("could not create share: " + share_name)
 
     def delete_share(self, share_name):
-        container_name = utils.make_share_name(share_name)
+        container_name = store_utils.make_share_name(share_name)
         self._check_ws_name(container_name)
 
         result = self.provider.delete_container(container_name)    
@@ -623,6 +644,9 @@ class StoreBlobObjs():
                  errors.store_error("Workspace not found: {}".format(ws_name))
             self.create_workspace(ws_name)
 
+        created = not exists
+        return created
+
     def create_workspace(self, ws_name, description=None):
         ''' create workspace as top level container '''
         self._check_ws_name(ws_name)
@@ -640,31 +664,31 @@ class StoreBlobObjs():
 
         # MULTIPROCESS: safe now
 
-        # create a holder file for RUNS directory
-        runs_holder_fn = constants.RUNS_DIR + "/" + constants.HOLDER_FILE
-        self._create_blob(ws_name, runs_holder_fn, "1", True)
+        # create a holder file for JOBS directory
+        holder_fn = constants.JOBS_DIR + "/" + constants.HOLDER_FILE
+        self._create_blob(ws_name, holder_fn, "1", True)
 
         # create a holder file for EXPERIMENTS directory
         experiments_holder_fn = constants.EXPERIMENTS_DIR + "/" + constants.HOLDER_FILE
         self._create_blob(ws_name, experiments_holder_fn, "1", True)
 
-        # create NEXT_RUN_NAME (for extra safety, ensure file doesn't already exist)
-        blob_fn = constants.WORKSPACE_DIR + "/" + constants.WORKSPACE_NEXT
-        self._create_blob(ws_name, blob_fn, "1", True)
-
+        # create LAST JOB CREATED blob (for extra safety, ensure file doesn't already exist)
+        blob_fn = constants.WORKSPACE_DIR + "/" + constants.WORKSPACE_LAST_JOB
+        self._create_blob(ws_name, blob_fn, "0", True)
 
     def delete_workspace(self, ws_name):
         result = self.provider.delete_container(ws_name)    
-
         if not result:
              errors.store_error("could not delete workspace: " + ws_name)
 
         return result
 
     def get_workspace_names(self):
-        containers = self.provider.list_containers()
 
-        names = [name for name in containers if self.workspace_files(name, use_blobs=True).does_file_exist(constants.WORKSPACE_NEXT)]
+        # legacy - mongo.get_workspace_names is now preferred
+        containers = self.provider.list_containers()
+        names = [name for name in containers if self.workspace_files(name, use_blobs=True).does_file_exist(constants.WORKSPACE_LAST_JOB)]
+       
         return names
 
     # ---- EXPERIMENTS ----
@@ -701,6 +725,7 @@ class StoreBlobObjs():
         #console.print("ws_name=", ws_name, ", blob_path=", blob_path)
         return self.provider.does_blob_exist(ws_name, blob_path)
 
+    # NOTE: this is obsolete (any callers?)
     def get_run_names(self, ws_name):
         # enumerate all blobs in workspace (ouch)
         run_names = self.provider.list_blobs(ws_name, path=constants.RUNS_DIR + "/run*", recursive=False)
@@ -762,29 +787,6 @@ class StoreBlobObjs():
 
         return lines
 
-    # def create_run_directory(self, ws_name, run_name):
-    #     pass   # nothing to do for blobs here
-
-    def get_legacy_next_run_id(self, ws_name):
-         # get next run number
-        fn_next = constants.WORKSPACE_DIR + "/" + constants.WORKSPACE_NEXT
-        #console.print("reading next num, fn_next=", fn_next)
-
-        next = None
-        if self.provider.does_blob_exist(ws_name, fn_next):
-            text = self._read_blob(ws_name, fn_next)
-            next = int(text)
-
-        return next
-
-    def remove_legacy_run_control_file(self, ws_name):
-        fn_next = constants.WORKSPACE_DIR + "/" + constants.WORKSPACE_NEXT
-        #console.print("reading next num, fn_next=", fn_next)
-
-        if self.provider.does_blob_exist(ws_name, fn_next):
-            self.provider.delete_blob(ws_name, fn_next)
-
-
     def create_next_run_by_name(self, ws_name, run_name):
         '''
         NOTE: unique run_name has already been assigned thru MONGO atomic process.
@@ -804,58 +806,65 @@ class StoreBlobObjs():
         dest_path = self._make_run_path_fn(to_run, to_path)
         return self._copy_files(ws_name, from_path, run_wildcard, dest_path)
 
-    def read_legacy_next_job_id(self):
-        # read next job number from control file
-        info_dir = constants.INFO_DIR
-        fn_next = info_dir + "/" + constants.JOBS_NEXT
-        next_id = None
+    # def read_legacy_next_job_id(self):
+    #     # read next job number from control file
+    #     info_dir = constants.INFO_DIR
+    #     fn_next = info_dir + "/" + constants.JOBS_NEXT
+    #     next_id = None
 
-        if self.provider.does_blob_exist(constants.INFO_CONTAINER, fn_next):
-            text = self._read_blob(constants.INFO_CONTAINER, fn_next)
-            next_id = int(text) if text else 1
-        return next_id
+    #     if self.provider.does_blob_exist(constants.INFO_CONTAINER, fn_next):
+    #         text = self._read_blob(constants.INFO_CONTAINER, fn_next)
+    #         next_id = int(text) if text else 1
+    #     return next_id
 
     #---- JOBS ----
 
-    def read_job_info_file(self, job_name):
+    def read_job_info_file(self, ws_name, job_name):
         fn_job = self._get_job_path_fn(job_name, constants.JOB_INFO_FN)
-        if self.provider.does_blob_exist(constants.INFO_CONTAINER, fn_job):
-            text = self._read_blob(constants.INFO_CONTAINER, fn_job)
+        if self.provider.does_blob_exist(ws_name, fn_job):
+            text = self._read_blob(ws_name, fn_job)
         else:
             text = ""
 
         return text
 
-    def write_job_info_file(self, job_name, text):
+    def write_job_info_file(self, ws_name, job_name, text):
         fn_job = self._get_job_path_fn(job_name, constants.JOB_INFO_FN)
-        self._create_blob(constants.INFO_CONTAINER, fn_job, text)
+        self._create_blob(ws_name, fn_job, text)
 
-    def create_job_dir(self, job_name):
+    def does_job_exist(self, ws_name, job_id):
+        fn_job_info = self._get_job_path_fn(job_id, constants.JOB_INFO_FN)
+        return self.provider.does_blob_exist(ws_name, fn_job_info)
+        
+    def create_job_dir(self, ws_name, job_name):
         #self.create_info_container_if_needed()
         job_dir_path = self._get_job_path_fn(job_name, constants.HOLDER_FILE)
 
         # MULTIPROCESS: safe, because only 1 user will try to create the job dir (?)
-        if not self.provider.does_blob_exist(constants.INFO_CONTAINER, job_dir_path):
-            self._create_blob(constants.INFO_CONTAINER, job_dir_path, "1")
+        if not self.provider.does_blob_exist(ws_name, job_dir_path):
+            self._create_blob(ws_name, job_dir_path, "1")
 
-    def get_job_names(self):
+    def get_job_names(self, ws_name):
         path = constants.JOBS_DIR 
-        names = [ name[5:-1] for name in self.provider.list_blobs(constants.INFO_CONTAINER, path=constants.JOBS_DIR + "/job*", recursive=False) ]
+        names = [ name[5:-1] for name in self.provider.list_blobs(ws_name, path=constants.JOBS_DIR + "/job*", recursive=False) ]
         #console.print("names=", names)
          
+        # filter out imported and other wierd job names
+        names = [ name for name in names if name.startswith("job") and not "_" in name ]
+
         # sort by increasing job number
         names.sort(key=lambda name: int(name[3:]))
         return names
 
-    def append_job_run_name(self, job_name, run_name):
+    def append_job_run_name(self, ws_name, job_name, run_name):
         path = self._get_job_path_fn(job_name, constants.AGGREGATED_RUN_NAMES_FN)
-        self._append_blob(constants.INFO_CONTAINER, path, run_name + "\n", append_with_rewrite=True)
+        self._append_blob(ws_name, path, run_name + "\n", append_with_rewrite=True)
 
-    def get_job_run_names(self, job_name):
+    def get_job_run_names(self, ws_name, job_name):
         #console.print("getting job run_names for", job_name)
         path = self._get_job_path_fn(job_name, constants.AGGREGATED_RUN_NAMES_FN)
-        if self.provider.does_blob_exist(constants.INFO_CONTAINER, path):
-            text = self._read_blob(constants.INFO_CONTAINER, path)
+        if self.provider.does_blob_exist(ws_name, path):
+            text = self._read_blob(ws_name, path)
             text = text[:-1]    # remove last \n char
             run_names = text.split("\n")
         else:
@@ -895,11 +904,11 @@ class StoreBlobObjs():
         else:
             return store_azure_file.WorkspaceFiles(self, ws_name)
 
-    def job_files(self, job_name, use_blobs=False):
+    def job_files(self, ws_name, job_name, use_blobs=False):
         if use_blobs:
-            return JobBlobs(self, job_name)
+            return JobBlobs(self, ws_name, job_name)
         else:
-            return store_azure_file.JobFiles(self, job_name)
+            return store_azure_file.JobFiles(self, ws_name, job_name)
 
     def root_files(self, root_name, use_blobs=False):
         if use_blobs:
@@ -943,8 +952,9 @@ class BlobsObject():
         return self.store._append_blob(self.container, path, text)
 
     def read_file(self, fn):
-        path = self._expand_path(fn)
-        return self.store._read_blob(self.container, path)
+        #path = self._expand_path(fn)
+        container, path, wc_target = self._get_container_path_target(fn)
+        return self.store._read_blob(container, path)
 
     def upload_file(self, fn, source_fn, progress_callback=None):
         #path = self._expand_path(fn)
@@ -994,15 +1004,20 @@ class BlobsObject():
 
         if path:
             if path.startswith("$data"):
-
                 # expand $data
-                path = path.replace("$data", "/" + utils.DATA_STORE_ROOT)
+                path = path.replace("$data", "/" + store_utils.DATA_STORE_ROOT)
+
             elif path.startswith("$models"):
-
                 # expand $models
-                path = path.replace("$models", "/" + utils.MODELS_STORE_ROOT)
-            elif path.startswith("$runs"):
+                path = path.replace("$models", "/" + store_utils.MODELS_STORE_ROOT)
 
+            elif path.startswith("$jobs"):
+                # expand $jobs to: /workspace/jobs
+                job_prefix = "/{}/jobs".format(self.container)
+                path = path.replace("$jobs", job_prefix)
+
+            # TODO: what does $runs mean in v2?  maybe $run23 works better...
+            elif path.startswith("$runs"):
                 # expand $runs
                 self.base_path = "runs"
                 path = path.replace("$runs", "")
@@ -1165,7 +1180,9 @@ class RunBlobs(BlobsObject):
     def __init__(self, store, ws_name, run_name):
         self.store = store
         self.container = ws_name
-        self.base_path = constants.RUNS_DIR + "/" + run_name
+
+        parent_num = run_helper.get_parent_run_number(run_name)
+        self.base_path = "jobs/job{}/runs/{}".format(parent_num, run_name)
 
 class ExperimentBlobs(BlobsObject):
     def __init__(self, store, ws_name, exper_name):
@@ -1174,7 +1191,7 @@ class ExperimentBlobs(BlobsObject):
         self.base_path = constants.EXPERIMENTS_DIR + "/" + exper_name
 
 class JobBlobs(BlobsObject):
-    def __init__(self, store, job_name):
+    def __init__(self, store, ws_name, job_name):
         self.store = store
-        self.container = constants.INFO_CONTAINER
+        self.container = ws_name
         self.base_path = constants.JOBS_DIR + "/" + job_name
