@@ -9,10 +9,10 @@ import time
 import logging
 import subprocess
 
-from xtlib.storage.store import Store
 from xtlib import utils
 from xtlib import file_utils
 from .console import console
+from xtlib.storage import store
 
 logger = logging.getLogger(__name__)
 
@@ -29,7 +29,7 @@ class TensorboardReader():
 
         os.chdir(cwd)
 
-        self.store = Store.create_from_props_dict(store_props_dict)
+        self.store = store.create_from_props_dict(store_props_dict)
         self.print_progress = False
 
     def create_local_fn_from_template(self, run_record, ws_name, run_name, blob_name):
