@@ -20,6 +20,7 @@ import importlib
 from xtlib import errors
 from xtlib import constants
 from xtlib import file_utils
+from xtlib import job_helper
 
 from xtlib.console import console
 from xtlib.helpers.feedbackParts import feedback as fb
@@ -63,8 +64,11 @@ def is_azure_ml_box(box_name):
 def is_philly_box(box_name):
     return (box_name and "-philly-" in box_name)
 
+def is_itp_box(box_name):
+    return (box_name and "-itp-" in box_name)
+
 def is_service_box(box_name):
-    return box_name and box_name.startswith("job") and "-" in box_name
+    return box_name and job_helper.is_job_id(box_name) and "-" in box_name
 
 def load_json_records(text):
     # each line is a JSON text record, with a newline at the end
@@ -278,6 +282,15 @@ def get_time():
     #return time.time()
     return str(arrow.now()) 
 
+def parse_time(arrow_time_str):
+    '''
+    parse an datetime string produced by arrow module
+    and return value in form of datetime object. 
+    '''
+    atime = arrow.get(arrow_time_str)
+    dt = atime.datetime
+    return dt
+
 def is_philly_job(job_info):
     is_philly = False
     
@@ -292,6 +305,27 @@ def is_philly_job(job_info):
 def safe_value(dd, key, default=None):
     return dd[key] if dd and key in dd else default
 
+def safe_delete(dd, key, default=None):
+    value = default
+    if key in dd:
+        value = dd[key]
+        del dd[key]
+
+    return value
+
+def safe_move(newdd, dd, key, default=None, flatten=False):
+    value = default
+    if key in dd:
+        value = dd[key]
+        del dd[key]
+
+        if flatten:
+            newdd.update(value)
+        else:
+            newdd[key] = value
+
+    return value
+
 def safe_nested_value(dd, key, default=None):
     parts = key.split(".")
     value = dd
@@ -311,15 +345,6 @@ def make_box_name(job_id, service_name, node_index):
     box_name = "{}-{}-{}".format(job_id, service_name, node_index)
     return box_name
 
-def make_share_name(name):
-    return "xts-{}-xts".format(name)
-
-def is_share_name(name):
-    return name and name.startswith("xts-") and name.endswith("-xts")
-
-MODELS_STORE_ROOT = make_share_name("models")
-DATA_STORE_ROOT = make_share_name("data")
-
 def get_provider_code_path_from_context(context, provider_type, name):
     '''
     return the class constructor method for the specified provider.
@@ -405,13 +430,16 @@ def copy_to_submit_logs(args, fn, fnx=None):
 
 def get_controller_cwd(is_windows, is_local=False):
     if is_windows:
+        # we only support windows as a local machine
+        #cwd = os.path.expanduser("~/.xt/cwd")
+
         # docker has problems mapping paths to user home directories (~/)
         # controller app has problems copying/deleting files in 'programdata' folder
-        # so, for windows, we use this
-        program_data = "c:\\xt_controller"
-        cwd = file_utils.path_join(program_data, "xt", "cwd")
+        # so, for windows, we use this:
+        home_drive = os.getenv("HOMEDRIVE")
+        cwd = home_drive + "\\.xt\\cwd"
     else:   
-        cwd =  "~/.xt/cwd"
+        cwd = "~/.xt/cwd"
 
         # only safe to expand if local
         if is_local:
@@ -514,3 +542,4 @@ def find_by_property(elements, prop, target):
 def report_elapsed(start, name):
     elapsed = time.time() - start
     print("{} took: {:.2f} secs".format(name, elapsed))
+
